# -*- coding: utf-8 -*-
"""
This module belongs to Lauetools project

JS Micha Feb 2012

http://sourceforge.net/projects/lauetools/


#   this module contains general functions to index and refine data and
#   calibrate detector (on top geometry)

"""
import os
import sys
import time as ttt

import scipy.io
import numpy as np

if sys.version_info.major == 3:
    from . import LaueGeometry as F2TC
    from . import indexingAnglesLUT as INDEX
    from . import indexingSpotsSet as ISS
    from . import lauecore as LAUE
    from . import FitOrient as FitO
    from . import readmccd as rmccd
    from . import IOLaueTools as RWASCII
    from . import generaltools as GT
    from . import CrystalParameters as CP
    from . import dict_LaueTools as DictLT
    from . matchingrate import SpotLinks
else:
    import LaueGeometry as F2TC
    import indexingAnglesLUT as INDEX
    import indexingSpotsSet as ISS
    import lauecore as LAUE
    import FitOrient as FitO
    import readmccd as rmccd
    import IOLaueTools as RWASCII
    import generaltools as GT
    import CrystalParameters as CP
    from matchingrate import SpotLinks
    import dict_LaueTools as DictLT

CST_ENERGYKEV = DictLT.CST_ENERGYKEV


def Non_Indexed_Spots(indexed_spots, n):
    # dictionary of exp spots
    list_nonindexed = []
    for k in range(n):
        if indexed_spots[k][-1] == 0:
            list_nonindexed.append(k)
    return list_nonindexed

def SimulateResult_withMiller(grain, emin, emax, defaultParam, pixelsize=165.0 / 2048):
    """Simulate Laue Pattern for 1 grain

    Returns Twicetheta, Chi, Miller_ind, posx, posy, Energy arrays

    #TODO : use better function from laue6.py
    """
    detectordiameter = 165.0

    # if fastcompute = 0 array(indices) = 0 and TwicethetaChi is a list of spot object
    # k_direction = (Z>0)
    # TODO: to be argument if camera is far from preset kf_direction!!

    key_material = grain[3]
    grain = CP.Prepare_Grain(key_material, grain[2])

    # array(vec) and array(indices)  of spots exiting the crystal in 2pi steradian (Z>0)
    Spots2pi = LAUE.getLaueSpots(CST_ENERGYKEV / emax,
                                CST_ENERGYKEV / emin,
                                [grain],
                                fastcompute=0,
                                verbose=0,
                                kf_direction="Z>0")

    # 2theta, chi of spot which are on camera (with harmonics)
    TwicethetaChi = LAUE.filterLaueSpots(Spots2pi,
                                        fileOK=0,
                                        fastcompute=0,
                                        detectordistance=defaultParam[0],  # TODO: check if it is really the parameter set we need
                                        detectordiameter=detectordiameter * 1.2,
                                        kf_direction="Z>0",
                                        HarmonicsRemoval=1,
                                        pixelsize=pixelsize)

    Twicetheta = [spot.Twicetheta for spot in TwicethetaChi[0]]
    Chi = [spot.Chi for spot in TwicethetaChi[0]]
    Miller_ind = [list(spot.Millers) for spot in TwicethetaChi[0]]
    Energy = [spot.EwaldRadius * CST_ENERGYKEV for spot in TwicethetaChi[0]]

    posx, posy = F2TC.calc_xycam_from2thetachi(Twicetheta,
                                                Chi,
                                                defaultParam,
                                                verbose=0,
                                                pixelsize=pixelsize)[:2]

    return Twicetheta, Chi, Miller_ind, posx, posy, Energy


def simulate_theo(grain, emax, emin, paramDet, pixelsize, dim):
    """
    simulate theo for calibration purpose

    TODO: merge with SimulateResult_withMiller()
    """

    detectordistance = paramDet[0]
    detectordiameter = dim[0] * pixelsize * 3.0
    print("LAA.simulate_theo detectordiameter = ", detectordiameter)
    if abs(paramDet[3]) > 5.0:
        detectordiameter = 4.0 * detectordiameter
        print("detector diameter = ", detectordiameter)
        print("large xbet angle for diamond camera at 2theta = 118 deg")
        print("detectordiameter increased in LaueAutoAnalysis.py, def simulate_theo")
        print("in order to index all spots")

    kf_direction = "Z>0"

    key_material = grain[3]
    grain = CP.Prepare_Grain(key_material, grain[2])

    # array(vec) and array(indices)  of spots exiting the crystal in 2pi steradian (Z>0)
    spots2pi = LAUE.getLaueSpots(CST_ENERGYKEV / emax,
                                CST_ENERGYKEV / emin,
                                [grain],
                                fastcompute=0,
                                verbose=0,
                                kf_direction=kf_direction)

    # 2theta, chi of spot which are on camera (with harmonics)
    TwicethetaChi = LAUE.filterLaueSpots(spots2pi,
                                        fileOK=0,
                                        fastcompute=0,
                                        detectordistance=detectordistance,
                                        detectordiameter=detectordiameter,
                                        kf_direction=kf_direction)

    twicetheta = [spot.Twicetheta for spot in TwicethetaChi[0]]
    chi = [spot.Chi for spot in TwicethetaChi[0]]
    Miller_ind = [list(spot.Millers) for spot in TwicethetaChi[0]]

    posx, posy = F2TC.calc_xycam_from2thetachi(twicetheta,
                                                chi,
                                                paramDet[:5],
                                                verbose=0,
                                                pixelsize=pixelsize)[:2]

    return twicetheta, chi, Miller_ind, posx, posy


def RefineUB(linkedspots_link,
                linkExpMiller_link,
                linkIntensity_link,
                CCDcalib,
                data,
                starting_orientmatrix,
                Bmatrix,
                pixelsize=165.0 / 2048,
                dim=(2048, 2048),
                use_weights=True):
    """
    see OnRefinePicky in class picky_frame of LaueTool24.py

    CCDcalib = geometric detector parameters
    data = (2*select_theta, select_chi, select_I, DataPlot_filename)

    UP TO NOW: only strain and orientation simultaneously
    """
    linkedspots_fit = linkedspots_link
    linkExpMiller_fit = linkExpMiller_link
    linkIntensity_fit = linkIntensity_link
    linkResidues_fit = None

    print("\nStarting fit of strain and orientation from spots links ...\n")
    # print "Pairs of spots used",self.linkedspots
    arraycouples = np.array(linkedspots_fit)

    exp_indices = np.array(arraycouples[:, 0], dtype=np.int)
    sim_indices = np.array(arraycouples[:, 1], dtype=np.int)

    nb_pairs = len(exp_indices)
    print("Nb of pairs: ", nb_pairs)

    # print "exp_indices, sim_indices",exp_indices, sim_indices

    # self.data_theo contains the current simulated spots: twicetheta, chi, Miller_ind, posx, posy
    # Data_Q = self.data_theo[2]  # all miller indices must be entered with sim_indices = arraycouples[:,1]

    # print "self.linkExpMiller",self.linkExpMiller
    Data_Q = np.array(linkExpMiller_fit)[:, 1:]
    sim_indices = np.arange(nb_pairs)  # for fitting function this must be an arange...
    # print "DataQ from self.linkExpMiller",Data_Q

    # initial parameters of calibration ----------------------
    calib = CCDcalib
    # print "detector parameters", calib

    # experimental spots selection -------------------------------------

    _twth, _chi = (np.take(data[0], exp_indices),
                    np.take(data[1], exp_indices))  # 2theta chi coordinates
    # pixel coordinates
    pixX, pixY, _th = F2TC.calc_xycam_from2thetachi(
        _twth, _chi, calib, verbose=0, pixelsize=pixelsize)

    # print "2*_th must be equal to _twth",(2*_th-_twth) < 0.000001 # this a test

    # print "nb_pairs",nb_pairs
    # print "indices of simulated spots (selection in whole Data_Q list)",sim_indices
    # print "Experimental pixX, pixY",pixX, pixY
    # print "starting_orientmatrix",starting_orientmatrix

    if use_weights:
        weights = linkIntensity_fit
    else:
        weights = None

    results = None
    # fitresults = False

    if 1:  # fitting procedure for one or many parameters

        calib = list(calib)

        initial_values = np.array([1.0, 1.0, 0.0, 0.0, 0.0, 0, 0.0, 0.0])
        allparameters = np.array(calib + [1, 1, 0, 0, 0] + [0, 0, 0])

        # nspots = np.arange(nb_pairs)

        arr_indexvaryingparameters = np.arange(5, 13)
        # print "\nInitial error--------------------------------------\n"
        residues, deltamat, _ = FitO.error_function_on_demand_strain(initial_values,
                                                                        Data_Q,
                                                                        allparameters,
                                                                        arr_indexvaryingparameters,
                                                                        sim_indices,
                                                                        pixX,
                                                                        pixY,
                                                                        initrot=starting_orientmatrix,
                                                                        Bmat=Bmatrix,
                                                                        pureRotation=0,
                                                                        verbose=1,
                                                                        pixelsize=pixelsize,
                                                                        dim=dim,
                                                                        weights=weights)
        # print "Initial residues",residues
        # print "---------------------------------------------------\n"

        results = FitO.fit_on_demand_strain(initial_values,
                                                Data_Q,
                                                allparameters,
                                                FitO.error_function_on_demand_strain,
                                                arr_indexvaryingparameters,
                                                sim_indices,
                                                pixX,
                                                pixY,
                                                initrot=starting_orientmatrix,
                                                Bmat=Bmatrix,
                                                pixelsize=pixelsize,
                                                dim=dim,
                                                verbose=0,
                                                weights=weights)

        # print "\n********************\n       Results of Fit        \n********************"
        # print "results",results

    if results != None:

        # fitresults = True

        # print "\nFinal error--------------------------------------\n"
        residues, deltamat, _ = FitO.error_function_on_demand_strain(results,
                                                                        Data_Q,
                                                                        allparameters,
                                                                        arr_indexvaryingparameters,
                                                                        sim_indices,
                                                                        pixX,
                                                                        pixY,
                                                                        initrot=starting_orientmatrix,
                                                                        Bmat=Bmatrix,
                                                                        pureRotation=0,
                                                                        verbose=1,
                                                                        pixelsize=pixelsize,
                                                                        dim=dim,
                                                                        weights=weights)

        residues_non_weighted = FitO.error_function_on_demand_strain(results,
                                                        Data_Q,
                                                        allparameters,
                                                        arr_indexvaryingparameters,
                                                        sim_indices,
                                                        pixX,
                                                        pixY,
                                                        initrot=starting_orientmatrix,
                                                        Bmat=Bmatrix,
                                                        pureRotation=0,
                                                        verbose=1,
                                                        pixelsize=pixelsize,
                                                        dim=dim,
                                                        weights=None)[0]

        # print "Final residues",residues
        # print "---------------------------------------------------\n"
        # print "mean",np.mean(residues)

        # building B mat
        param_strain_sol = results
        varyingstrain = np.array([[1.0, param_strain_sol[2], param_strain_sol[3]],
                                    [0, param_strain_sol[0], param_strain_sol[4]],
                                    [0, 0, param_strain_sol[1]]])
        # print "varyingstrain results"
        # print varyingstrain

        # building UBmat (= newmatrix)
        UBmat = np.dot(np.dot(deltamat, starting_orientmatrix), varyingstrain)
        # print "UBmat",UBmat
        # print "newmatrix",newmatrix # must be equal to UBmat

        RR, PP = GT.UBdecomposition_RRPP(UBmat)

        # symetric matrix (strain) in direct real distance
        epsil = GT.epsline_to_epsmat(CP.calc_epsp(CP.dlat_to_rlat(CP.matrix_to_rlat(PP))))
        # print "epsil is already a zero trace symetric matrix ",np.round(epsil*1000, decimals = 2)
        # if the trace is not zero
        epsil = epsil - np.trace(epsil) * np.eye(3) / 3.0

        epsil = np.round(epsil * 1000, decimals=2)
        print("\n********************\n       result of Strain and Orientation  Fitting        \n********************")
        # print "last pixdev table non weighted"
        # print residues_non_weighted.round(decimals=3)
        print("Mean pixdev non weighted")
        pixdev = np.mean(residues_non_weighted).round(decimals=5)
        print(pixdev)
        print("\nPure Orientation Matrix")
        # print RR.tolist()
        print(RR)

        print("Deviatoric strain with respect to initial crystal unit cell (in 10-3 units)")
        # print epsil.tolist()
        print(epsil)

        # pas = np.array([[0,-1, 0],[0, 0, -1],[1, 0, 0]])
        # invpas = np.linalg.inv(pas)
        # print "good deviatoric strain epsil"  # ???? in other frame like XMAS
        # print np.dot(pas, epsil) # ????

        # update linked spots with residues

        linkResidues_fit = np.array([exp_indices, sim_indices, residues]).T

        # for further use
        newUBmat = UBmat
        newUmat = np.dot(deltamat, starting_orientmatrix)
        newBmat = varyingstrain
        deviatoricstrain = epsil

        return (newUBmat, newUmat, newBmat, deviatoricstrain, RR, linkResidues_fit, pixdev)


def GetStrainOrient(orientmatrix, Bmatrix, elem_label, emax_simul, veryclose_angletol, data, addoutput=0,
                    saturated=None,
                    use_weights=True,
                    addoutput2=0,
                    Emin=5.0,
                    detectorparameters=None,  # dico cf dans test_index_refine
                ):

    """
    refine UB

    returns:

    newUBmat : refined UB (that can be used later for simulation or more refinement
    deviatoricstrain:
    RR: pure rotation
    linkResidues_fit:  spots indices that have been used

    saturated = sat_flag list : to exclude saturated peaks from refining of UB

    """

    # modif 10Dec14

    #    vecteurref = np.eye(3) # means: a* // X, b* // Y, c* //Z
    #    grain = [vecteurref, [1, 1, 1], orientmatrix, elem_label]

    # PATCH: redefinition of grain to simulate any unit cell (not only cubic) ---
    #    key_material = elem_label # grain[3]
    grain = CP.Prepare_Grain(elem_label, orientmatrix)  # grain[2])

    # attention SimulateLaue attend une OrientMatrix avec norme(astar) = 1
    #  dans la matrice UBB0 de JSM la matrice UB a norme(astar) = 1
    # c'est cette convention qui est utilisee pour le calcul de l'energie des spots
    # donc pas robuste par rapport aux permutations de a b c dans le cubique quand UB est deformee
    # -----------------------------------------------------------------------------

    # theoretical spots
    #    Twicetheta_theo, Chi_theo, Miller_ind_theo, posx_theo, posy_theo, energy_theo = \
    #
    #            SimulateResult_withMiller(grain, 5, emax_simul,
    #                                        CCDcalib,
    #                                        pixelsize=165. / 2048,
    #                                        dim=(2048, 2048)
    #                                        )
    Emax = emax_simul

    (Twicetheta_theo,
        Chi_theo,
        Miller_ind_theo,
        _,
        _,
        energy_theo,
    ) = LAUE.SimulateLaue(grain,
                            Emin,
                            Emax,
                            detectorparameters["detectorparameters"],  # calib
                            kf_direction=detectorparameters["kf_direction"],
                            removeharmonics=1,
                            pixelsize=detectorparameters["pixelsize"],
                            dim=detectorparameters["dim"],
                            ResolutionAngstrom=False,
                            detectordiameter=detectorparameters["detectordiameter"] * 1.25)

    # experimental_spots
    twicetheta_exp, chi_exp, dataintensity_exp = data[:3]

    #    print "len(twicetheta_exp)", len(twicetheta_exp)
    #    print "twicetheta_exp, chi_exp",twicetheta_exp, chi_exp
    #    print "len(twicetheta_exp)", len(Twicetheta_theo)
    #    print "Twicetheta_theo, Chi_theo",Twicetheta_theo, Chi_theo

    #    p.figure()
    #    ax = p.subplot(111)
    #    p.plot(twicetheta_exp, chi_exp, 'ro')
    #    p.plot(Twicetheta_theo, Chi_theo, 'bx')
    #    ax.axis("equal")
    #    klmsdf

    # build automatically theo and exp spots associations (see OnAutoLink(self, evt) of picky_Frame of LaueTool24.py)

    res_assoc = SpotLinks(twicetheta_exp,
                                        chi_exp,
                                        dataintensity_exp,  # experimental data
                                        veryclose_angletol,  # tolerance angle
                                        Twicetheta_theo,
                                        Chi_theo,
                                        Miller_ind_theo,
                                        energy_theo)

    # print "len(assoc)",len(res_assoc[0])
    # print res_assoc[0]

    if not res_assoc:
        return 0
    else:

        # refine UB
        (refine_indexed_spots, linkedspots_link, linkExpMiller_link, linkIntensity_link,
        _, _, _) = res_assoc

        npairs = np.shape(linkedspots_link)[0]
        if 0:
            print("linkedspots_link")
            print(linkedspots_link)
            print(np.shape(linkedspots_link))
            print("refine_indexed_spots")
            for key, value in refine_indexed_spots.items():
                print(key, value)

        if npairs < 8:
            print("Need at least 8 indexed spots to refine the unit cell !!")
            return None

        if saturated != None:
            # print " linkedspots_link ", linkedspots_link
            # print "linkExpMiller_link ", linkExpMiller_link
            # print "linkIntensity_link ", linkIntensity_link
            # print "shapes ", np.shape(linkedspots_link), np.shape(linkExpMiller_link), np.shape(linkIntensity_link)
            print("indexed exp spots : total : ", np.shape(linkedspots_link)[0])
            # indices par rapport a la peak list initiale (celle de data)
            indsat = np.where(saturated == 1)
            # print "indsat[0] ", indsat[0]
            sat_link = np.zeros(np.shape(linkedspots_link)[0], dtype=int)
            for i in indsat[0]:
                # print "i = ", i
                ind2 = np.where(linkedspots_link[:, 0] == i)
                # print "ind2[0] = ", ind2[0]
                if len(ind2[0]) == 1:
                    sat_link[ind2[0][0]] = 1
                else:
                    print("exp peak indexed %d times", len(ind2[0]))
                    raise ValueError("problem ?")
            # indices par rapport a la liste de pics indexes (celle de linkedspots_link)
            # print "sat_link", sat_link
            indnonsat = np.where(sat_link == 0)[0]
            # print "indnonsat = ", indnonsat
            linkedspots_link = linkedspots_link[indnonsat, :]
            linkExpMiller_link = np.array(linkExpMiller_link, dtype=int)[indnonsat, :]
            linkIntensity_link = np.array(linkIntensity_link, dtype=float)[indnonsat]
            # print " linkedspots_link ", linkedspots_link
            # print "new shapes ", np.shape(linkedspots_link), np.shape(linkExpMiller_link), np.shape(linkIntensity_link)
            print("indexed exp spots : non saturated :", np.shape(linkedspots_link)[0])

            # reconvertir linkExpMiller_link et linkIntensity_link en listes et pas array ?  => semble OK sans

        res_refinement = RefineUB(linkedspots_link,
                                    linkExpMiller_link,
                                    linkIntensity_link,
                                    detectorparameters["detectorparameters"],  # CCDcalib,
                                    data,
                                    orientmatrix,
                                    Bmatrix,
                                    pixelsize=detectorparameters["pixelsize"],
                                    dim=detectorparameters["dim"],
                                    use_weights=use_weights)

        newUBmat, newUmat, newBmat, deviatoricstrain, RR, linkResidues_fit, pixdev = res_refinement

        nb_fitted_peaks = len(linkedspots_link)
        print("with %d fitted peaks" % nb_fitted_peaks)
        print("UBmat", newUBmat)
        print("pure rotation ", RR)
        print("deviatoric strain", deviatoricstrain)

        if addoutput2:
            return (newUBmat,
                deviatoricstrain,
                RR,
                linkResidues_fit,
                pixdev,
                nb_fitted_peaks,
                linkExpMiller_link,
                linkedspots_link,
                linkIntensity_link)
        else:
            if addoutput:
                return (newUBmat,
                    deviatoricstrain,
                    RR,
                    linkResidues_fit,
                    pixdev,
                    nb_fitted_peaks,
                    linkExpMiller_link)

            else:
                return (newUBmat,
                    newUmat,
                    newBmat,
                    deviatoricstrain,
                    RR,
                    linkResidues_fit,
                    pixdev,
                    nb_fitted_peaks,
                    linkExpMiller_link)


def test_indexation_refinement_Ge():
    """
    from .dat file and calib parameters do indexation and refinement

    """
    dirname = "./Examples/Ge/"
    DataPlot_filename = "Ge0001.dat"
    # CCDcalib = [69.17369978864516611, 1050.50098288478648101, 1115.53514707010231177,
    #                 0.13049762505535042, -0.23799600745091942]
    pixelsize = 165.0 / 2048

    twicetheta, chi, dataintensity, data_x, data_y = F2TC.Compute_data2thetachi(
                                        os.path.join(dirname, DataPlot_filename),
                                        (0, 1, 3),
                                        1,  # 2 for centroid intensity, 3 for integrated  intensity
                                        sorting_intensity="yes",
                                        param=defaultParam,
                                        pixelsize=pixelsize)

    prefix, _ = DataPlot_filename.split(".")
    # writing .cor file
    RWASCII.writefile_cor(os.path.join(dirname, "C_" + prefix),
                            twicetheta,
                            chi,
                            data_x,
                            data_y,
                            dataintensity,
                            sortedexit=0,
                            param=defaultParam + [pixelsize],
                            initialfilename=DataPlot_filename)

    print("%s has been created with defaultparameter" % (prefix + ".cor"))
    print("%s" % str(defaultParam))
    # a C_#########.cor file is now created
    file_extension = "cor"
    DataPlot_filename = "C_" + prefix + "." + file_extension

    # Reading . cor file
    # TODO: put this function in readmccd for example
    # Current_peak_data = scipy.io.array_import.read_array(os.path.join(dirname, DataPlot_filename), lines=(1, -1))
    np.loadtxt(os.path.join(dirname, DataPlot_filename), skiprows=1)
    # nbcolumns == 5:
    data_theta = Current_peak_data[:, 0] / 2.0
    data_chi, data_pixX, data_pixY, data_I = np.transpose(Current_peak_data)[1:]

    # filename = DataPlot_filename

    # index_foundgrain = 0
    # dictionary of exp spots
    indexed_spots = {}
    for k in range(len(data_theta)):
        indexed_spots[k] = [k,  # index of experimental spot in .cor file
                            data_theta[k] * 2.0,
                            data_chi[k],  # 2theta, chi coordinates
                            data_pixX[k],
                            data_pixY[k],  # pixel coordinates
                            data_I[k],  # intensity
                            0]  # 0 means non indexed yet
    # last_orientmatrix_fromindexation = {}
    # last_Bmatrix_fromindexation = {}
    # last_epsil_fromindexation = {}

    # Running indexation

    # updated exp spot index to be still index
    current_exp_spot_index_list = Non_Indexed_Spots(indexed_spots, len(data_theta))

    # for each grain or indexation step: ******************************
    # ******************************************************************

    # compute angular distance between all exp; spots:

    nbspots_in_data = len(data_theta[current_exp_spot_index_list])

    # Matching spots set Size (MSSS):
    nbspotmaxformatching = nbspots_in_data

    # select 1rstly spots that have not been indexed and 2ndly reduced list by user
    index_to_select = np.take(current_exp_spot_index_list, np.arange(nbspotmaxformatching))

    select_theta = data_theta[index_to_select]
    select_chi = data_chi[index_to_select]
    select_I = data_I[index_to_select]
    # print select_theta
    # print select_chi
    listcouple = np.transpose(np.array([select_theta, select_chi]))
    Tabledistance = GT.calculdist_from_thetachi(listcouple, listcouple)

    # classical indexation parameters:
    data = (2 * select_theta, select_chi, select_I, DataPlot_filename)

    spot_index_central = [0, 1, 2, 3]  # central spot or list of spots
    nbmax_probed = 10  # 'Recognition spots set Size (RSSS): '
    elem_label = "UO2"
    energy_max = 25

    rough_tolangle = 0.5  # 'Dist. Recogn. Tol. Angle (deg)'
    fine_tolangle = 0.2  # 'Matching Tolerance Angle (deg)'
    Nb_criterium = 15  # 'Minimum Number Matched Spots: '
    print("rough_tolangle ", rough_tolangle)
    print("fine_tolangle ", fine_tolangle)
    NBRP = 2  # number of best result for each element of spot_index_central

    n = 3  # for LUT
    latticeparams = DictLT.dict_Materials[elem_label][1]
    B = CP.calc_B_RR(latticeparams)

    # indexation procedure
    bestmat, stats_res = INDEX.getOrientMatrices(spot_index_central,
                                                energy_max,
                                                Tabledistance[:nbmax_probed, :nbmax_probed],
                                                select_theta,
                                                select_chi,
                                                n,
                                                B,
                                                rough_tolangle=rough_tolangle,
                                                fine_tolangle=fine_tolangle,
                                                Nb_criterium=Nb_criterium,
                                                plot=0,
                                                structure_label=elem_label,
                                                nbbestplot=NBRP)

    # bestmat contains: NBRP * len(spot_index_central) matrix

    # Look for the very best matrix candidates: -----to be improved in readability ------------------------
    ar = []
    for elem in stats_res:
        ar.append([elem[0], -elem[2]])

    tabstat = np.array(ar)  # , dtype = [('x', '<i4'), ('y', '<i4')])
    rankmat = np.argsort(tabstat[:, 0])[::-1]  # ,order=('x','y'))

    verybestmat = bestmat[rankmat[0]]
    # -----------------------------------------------------------------------------------------------------
    # very best matrix
    orientmatrix = verybestmat

    # then get strain
    emax_simul = 16
    veryclose_angletol = 0.2

    (newUBmat, _, _, _,
    _, _, _, _) = GetStrainOrient(orientmatrix,
                                                                        B,
                                                                        elem_label,
                                                                        emax_simul,
                                                                        veryclose_angletol,
                                                                        data,
                                                                        detectorparameters=defaultParam,
                                                                        addoutput=0)
    # in this example the previous first step is useless
    emax_simul = 25
    veryclose_angletol = 0.1

    (_,
        _,
        _,
        deviatoricstrain_2,
        _,
        _,
        _,
        _,
    ) = GetStrainOrient(newUBmat, B, elem_label, emax_simul, veryclose_angletol,
                                                                    data,
                                                                    detectorparameters=defaultParam,
                                                                    addoutput=0)

    print("final strain")
    print(deviatoricstrain_2)


def RefineCalibParameters(linkedspots,
                            linkExpMiller,
                            linkIntensity,  # spots associations
                            data_x,
                            data_y,  # X, Y pixel exp data
                            matrix,
                            paramDet,
                            boolctrl,  # initial matrix guess, initial detector params, list of booleans for vary params
                            use_weights=True,
                            dim=(2048, 2048),
                            pixelsize=165.0 / 2048,
                            verbose=0,
                            addoutput=0):
    """
    fit detector parameters

    for varying parameters set True
    # dd, xcen, ycen, xbet, xgam a1,a2,a3
    boolctrl = '11000011'

    """

    arraycouples = np.array(linkedspots)

    exp_indices = np.array(arraycouples[:, 0], dtype=np.int)
    sim_indices = np.array(arraycouples[:, 1], dtype=np.int)

    nb_pairs = len(exp_indices)
    if verbose:
        print("Nb of pairs: ", nb_pairs)
        print(exp_indices, sim_indices)

    # self.data_theo contains the current simulated spots: twicetheta, chi, Miller_ind, posx, posy
    # Data_Q = self.data_theo[2]  # all miller indices must be entered with sim_indices = arraycouples[:,1]

    Data_Q = np.array(linkExpMiller)[:, 1:]

    sim_indices = np.arange(nb_pairs)
    if verbose:
        print("DataQ from self.linkExpMiller", Data_Q)
        print("self.linkExpMiller", linkExpMiller)

    # experimental spots selection from self.data_x, self.data_y (loaded when initialising calibFrame)
    pixX, pixY = (np.take(data_x, exp_indices), np.take(data_y, exp_indices))  # pixel coordinates
    # twth, chi = np.take(self.Data_X, exp_indices),np.take(self.Data_Y, exp_indices) # 2theta chi coordinates

    # initial parameters of calibration and misorientation from the current orientation matrix
    # print "Detector parameters in refine procedure",paramDet

    allparameters = np.array(paramDet + [0, 0, 0])  # 3 last params = 3 quaternion angles not used here

    # select the parameters that must be fitted
    # dd, xcen, ycen, xbet, xgam,  a1,a2,a3
    # boolctrl = '11001100'
    varyingparameters = []
    init_values = []
    for k, val in enumerate(boolctrl):
        if val == "1":
            varyingparameters.append(k)
            init_values.append(allparameters[k])

    if len(varyingparameters) == 0:
        print("You need to select at least one parameter to fit!!")
        return

    listparam = ["distance (mm)",
                "Xcen (pixel)",
                "Ycen (pixel)",
                "Angle1 (deg)",
                "Angle2 (deg)",  # detector parameter
                "theta1 (deg)",
                "theta2 (deg)",
                "theta3 (deg)"]  # misorientation with respect to initial matrix (/ elementary axis rotation)

    # start fit
    initial_values = np.array(init_values)  # [dd, xcen, ycen, ang1, ang2, theta1, theta2, theta3]
    arr_indexvaryingparameters = np.array(varyingparameters)  # indices of position of parameters in [dd, xcen, ycen, ang1, ang2, theta1, theta2, theta3]

    print("\n ***** \n starting fit of :", [listparam[k] for k in arr_indexvaryingparameters])
    print("With initial values: ", initial_values)
    if verbose:
        # print "miller selected ",np.take(self.data_theo[2],sim_indices, axis = 0) ????
        print("allparameters", allparameters)
        print("arr_indexvaryingparameters", arr_indexvaryingparameters)
        print("nb_pairs", nb_pairs)
        print("indices of simulated spots (selection in whole Data_Q list)", sim_indices)
        print("Experimental pixX, pixY", pixX, pixY)
        print("self.matrix", matrix)

    vecteurref = np.eye(3)  # use of Bmat will be handled later

    pureRotation = 1

    if use_weights:
        weights = linkIntensity
    else:
        weights = None

    # fitting procedure for one or many parameters
    nb_fittingparams = len(arr_indexvaryingparameters)
    if nb_pairs < nb_fittingparams:
        print("\n****************************************************************************")
        print("You need at least %d spots links to fit these %d parameters."
            % (nb_fittingparams, nb_fittingparams))
        print("****************************************************************************\n")
        return None, None, None

    # print "Initial error--------------------------------------\n"
    _, _, newmatrix = FitO.error_function_on_demand_calibration(initial_values,
                                                                        Data_Q,
                                                                        allparameters,
                                                                        arr_indexvaryingparameters,
                                                                        sim_indices,
                                                                        pixX,
                                                                        pixY,
                                                                        initrot=matrix,
                                                                        vecteurref=vecteurref,
                                                                        pureRotation=pureRotation,
                                                                        verbose=1,
                                                                        pixelsize=pixelsize,
                                                                        dim=dim,
                                                                        weights=weights)
    # print "Initial residues",residues
    # print "---------------------------------------------------\n"

    # print "Starting fit ------------"
    results = FitO.fit_on_demand_calibration(initial_values,
                                                Data_Q,
                                                allparameters,
                                                FitO.error_function_on_demand_calibration,
                                                arr_indexvaryingparameters,
                                                sim_indices,
                                                pixX,
                                                pixY,
                                                initrot=matrix,
                                                vecteurref=vecteurref,
                                                pureRotation=pureRotation,
                                                pixelsize=pixelsize,
                                                dim=dim,
                                                verbose=0,
                                                weights=weights)

    print("\n********************\n       Results of Fit        \n********************")
    # print "Fit Results",results
    allresults = allparameters

    if nb_fittingparams == 1:
        results = [results]

    # print "\n ----- Computation of fit results residues -------\n"

    # print "\n ----- weighted residues -------\n"
    _, _, newmatrix = FitO.error_function_on_demand_calibration(results,
                                                                        Data_Q,
                                                                        allparameters,
                                                                        arr_indexvaryingparameters,
                                                                        sim_indices,
                                                                        pixX,
                                                                        pixY,
                                                                        initrot=matrix,
                                                                        vecteurref=vecteurref,
                                                                        pureRotation=pureRotation,
                                                                        verbose=1,
                                                                        pixelsize=pixelsize,
                                                                        dim=dim,
                                                                        weights=weights)

    # print "residues",residues

    # print "\n ----- NON weighted residues -------\n"

    (residues_nonweighted,
    _delta, _newmatrix, _) = FitO.error_function_on_demand_calibration(
                                                                        results,
                                                                        Data_Q,
                                                                        allparameters,
                                                                        arr_indexvaryingparameters,
                                                                        sim_indices,
                                                                        pixX,
                                                                        pixY,
                                                                        initrot=matrix,
                                                                        vecteurref=vecteurref,
                                                                        pureRotation=pureRotation,
                                                                        verbose=1,
                                                                        pixelsize=pixelsize,
                                                                        dim=dim,
                                                                        weights=None,
                                                                        allspots_info=1,
                                                                    )

    # print "last pixdev table"
    # print residues_nonweighted
    print("Mean pixdev")
    print(np.mean(residues_nonweighted))
    # print "SpotsData"
    # print SpotsData
    # print "initial matrix"
    # print matrix
    # print "New delta matrix"
    # print deltamat
    # print "newmatrix"
    # print newmatrix
    # print newmatrix.tolist()

    if len(arr_indexvaryingparameters) > 1:
        for k, val in enumerate(arr_indexvaryingparameters):
            allresults[val] = results[k]
    elif len(arr_indexvaryingparameters) == 1:
        allresults[arr_indexvaryingparameters[0]] = results[0]

    residues_fit = residues_nonweighted

    dataresults = allresults.tolist() + [np.mean(residues_fit), len(residues_fit)]

    print("New parameters", allresults)

    # update orient matrix
    # print "updating orientation parameters"

    # matrix = newmatrix
    ##self.deltamatrix = np.eye(3) # identity
    # self.vecteurref is unchanged

    ## update exp data
    # twicetheta, chi, dataintensity = F2TC.Compute_data2thetachi(self.filename,
    # (0, 1, 3),
    # 0,
    # param = self.paramDet,
    # pixelsize = pixelsize,
    # dim = dim # only for peaks coming from fit2d doing an y direction inversion
    # )[:3]
    if addoutput:
        return allresults, newmatrix, dataresults, residues_nonweighted
    else:
        return allresults, newmatrix, dataresults


def test_peaksearch_index_refine():
    """
    data treatment sequence:

    peak search
    indexation
    strain refinement

    You may like to change slightly (to allow indexation) defaultParam in order to see how it does affect the resulting Ge strain

    """
    dirname = "./Examples/Ge/"
    imagefilename = "Ge0001.mccd"

    defaultParam = [69.17369978864516611, 1050.50098288478648101, 1115.53514707010231177, 0.13049762505535042, -0.23799600745091942]
    # defaultParam = [69.15, 1050, 1114, 0.1, -0.2]

    pixelsize = 165.0 / 2048

    # Automation of peaksearch + indexation + strain refinement

    # peak search
    time_0 = ttt.time()

    prefix = "autotest_Ge"
    Isorted, _, _ = rmccd.PeakSearch(os.path.join(dirname, imagefilename),
                                                            PixelNearRadius=5,
                                                            removeedge=2,
                                                            IntensityThreshold=500,
                                                            boxsize=5,
                                                            position_definition=1,
                                                            verbose=1,
                                                            fit_peaks_gaussian=1,
                                                            xtol=0.001,
                                                            return_histo=0,
                                                            FitPixelDev=2.0)

    filewritten = RWASCII.writefile_Peaklist(prefix,
                                            Isorted,
                                            position_definition=1,
                                            overwrite=1,
                                            initialfilename=imagefilename,
                                            comments="",
                                            dirname=dirname)  #'.')

    if filewritten:
        print("peak list written in %s" % (prefix + ".dat"))
        print("Working directory is now: ", dirname)
        print("execution time: %.2f sec" % (ttt.time() - time_0))
    else:
        raise ValueError("Please Change directory name")

    dat_filename = prefix + ".dat"

    twicetheta, chi, dataintensity, data_x, data_y = F2TC.Compute_data2thetachi(dat_filename,
                                                                            (0, 1, 3),
                                                                            0,
                                                                            sorting_intensity="yes",
                                                                            param=defaultParam,
                                                                            pixelsize=pixelsize)

    prefix, _ = dat_filename.split(".")
    # writing .cor file
    RWASCII.writefile_cor("C_" + prefix,
                        twicetheta,
                        chi,
                        data_x,
                        data_y,
                        dataintensity,
                        sortedexit=0,
                        param=defaultParam + [pixelsize],
                        initialfilename=dat_filename)

    print("%s has been created with defaultparameter" % (prefix + ".cor"))
    print("%s" % str(defaultParam))
    # a C_#########.cor file is now created
    file_extension = "cor"
    DataPlot_filename = "C_" + prefix + "." + file_extension

    # Reading . cor file
    # TODO use loadtxt or readcorfile somewhere
    Current_peak_data = scipy.io.array_import.read_array(DataPlot_filename, lines=(1, -1))

    # nbcolumns == 5:
    data_theta = Current_peak_data[:, 0] / 2.0
    data_chi, data_pixX, data_pixY, data_I = np.transpose(Current_peak_data)[1:]

    # filename = DataPlot_filename

    # index_foundgrain = 0
    # dictionary of exp spots
    indexed_spots = {}
    for k in range(len(data_theta)):
        indexed_spots[k] = [k,  # index of experimental spot in .cor file
                            data_theta[k] * 2.0,
                            data_chi[k],  # 2theta, chi coordinates
                            data_pixX[k],
                            data_pixY[k],  # pixel coordinates
                            data_I[k],  # intensity
                            0]  # 0 means non indexed yet
    # last_orientmatrix_fromindexation = {}
    # last_Bmatrix_fromindexation = {}
    # last_epsil_fromindexation = {}

    # Running indexation

    # updated exp spot index to be still index
    current_exp_spot_index_list = Non_Indexed_Spots(indexed_spots, len(data_theta))

    # for each grain or indexation step: ******************************
    # ******************************************************************

    # compute angular distance between all exp; spots:

    nbspots_in_data = len(data_theta[current_exp_spot_index_list])

    # Matching spots set Size (MSSS):
    nbspotmaxformatching = nbspots_in_data

    # select 1rstly spots that have not been indexed and 2ndly reduced list by user
    index_to_select = np.take(current_exp_spot_index_list, np.arange(nbspotmaxformatching))

    select_theta = data_theta[index_to_select]
    select_chi = data_chi[index_to_select]
    select_I = data_I[index_to_select]
    # print select_theta
    # print select_chi
    listcouple = np.transpose(np.array([select_theta, select_chi]))
    Tabledistance = GT.calculdist_from_thetachi(listcouple, listcouple)

    # classical indexation parameters:
    data = (2 * select_theta, select_chi, select_I, DataPlot_filename)

    spot_index_central = [0, 1, 2, 3]  # central spot or list of spots
    nbmax_probed = 10  # 'Recognition spots set Size (RSSS): '
    elem_label = "UO2"
    energy_max = 25

    rough_tolangle = 0.5  # 'Dist. Recogn. Tol. Angle (deg)'
    fine_tolangle = 0.2  # 'Matching Tolerance Angle (deg)'
    Nb_criterium = 15  # 'Minimum Number Matched Spots: '
    print("rough_tolangle ", rough_tolangle)
    print("fine_tolangle ", fine_tolangle)
    NBRP = 2  # number of best result for each element of spot_index_central

    n = 3  # for LUT
    latticeparams = DictLT.dict_Materials[elem_label][1]
    B = CP.calc_B_RR(latticeparams)

    # indexation procedure
    bestmat, stats_res = INDEX.getOrientMatrices(spot_index_central,
                                                    energy_max,
                                                    Tabledistance[:nbmax_probed, :nbmax_probed],
                                                    select_theta,
                                                    select_chi,
                                                    n,
                                                    B,
                                                    rough_tolangle=rough_tolangle,
                                                    fine_tolangle=fine_tolangle,
                                                    Nb_criterium=Nb_criterium,
                                                    plot=0,
                                                    structure_label=elem_label,
                                                    nbbestplot=NBRP)

    # bestmat contains: NBRP * len(spot_index_central) matrix

    # Look for the very best matrix candidates: -----to be improved in readability ------------------------
    ar = []
    for elem in stats_res:
        ar.append([elem[0], -elem[2]])

    tabstat = np.array(ar)  # , dtype = [('x', '<i4'), ('y', '<i4')])
    rankmat = np.argsort(tabstat[:, 0])[::-1]  # ,order=('x','y'))

    verybestmat = bestmat[rankmat[0]]
    # -----------------------------------------------------------------------------------------------------
    # very best matrix
    orientmatrix = verybestmat

    # the resimulate to output miller indices
    emax_simul = 16
    veryclose_angletol = 0.2

    Bmatrix = np.eye(3)
    (newUBmat, _, _, _,
    _, _, _, _) = GetStrainOrient(orientmatrix,
                                                                    Bmatrix,
                                                                    elem_label,
                                                                    emax_simul,
                                                                    veryclose_angletol,
                                                                    data,
                                                                    detectorparameters=defaultParam,
                                                                    addoutput=0)

    emax_simul = 25
    veryclose_angletol = 0.1

    GetStrainOrient(newUBmat,
                    Bmatrix,
                    elem_label,
                    emax_simul,
                    veryclose_angletol,
                    data,
                    detectorparameters=defaultParam,
                    addoutput=0)


def GetCalibParameter(emax, veryclose_angletol, element, startingmatrix, defaultParam, data_exp,
                                                                        pixelsize=165.0 / 2048,
                                                                        dim=(2048, 2048),
                                                                        boolctrl="1" * 8,
                                                                        use_weights=True,
                                                                        addoutput=0):
    """
    data_exp = twicetheta, chi, dataintensity
    to allow dd, xcen, ycen xbet, xgam, three disorientation angles of crystal
    boolctrl = '11000110'
    """

    matrix = startingmatrix
    Grain = [np.eye(3), [1, 1, 1], matrix, element]

    paramDet = defaultParam
    print("paramDet in LAA.GetCalibParameters :", paramDet)
    if not isinstance(paramDet, list):
        paramDet = paramDet.tolist()

    # simulated theo spots
    data_theo = simulate_theo(Grain, emax, 5, paramDet, pixelsize, dim)
    Twicetheta_theo, Chi_theo, Miller_ind_theo, _, _ = data_theo

    # exp data
    twicetheta, chi, dataintensity, data_x, data_y = data_exp

    # build automatically theo and exp spots associations (see OnAutoLink(self, evt) of picky_Frame of LaueTool24.py)
    energy_theo = 10 * np.ones(len(Twicetheta_theo))
    res_assoc = SpotLinks(twicetheta,
                            chi,
                            dataintensity,  # experimental data
                            veryclose_angletol,  # tolerance angle
                            Twicetheta_theo,
                            Chi_theo,
                            Miller_ind_theo,
                            energy_theo)

    if not res_assoc:
        return None
    else:
        (_,
            linkedspots_link,
            linkExpMiller_link,
            linkIntensity_link,
            _,
            _,
            _,
        ) = res_assoc

        linkedspots = linkedspots_link
        linkExpMiller = linkExpMiller_link
        linkIntensity = linkIntensity_link
        # linkResidues = linkResidues_link

        nb_of_pairs = len(linkedspots)

        # test nb of varying params / nb pairs
        nbvaryingparams = boolctrl.count("1")

        if nbvaryingparams > nb_of_pairs:
            print("There are too few spots pairs to perform the fitting procedure")
            print("nbvaryingparams", nbvaryingparams)
            print("nb of pairs", nb_of_pairs)

        newparam, newmat, dr, pixdevlist = RefineCalibParameters(linkedspots,
                                                                linkExpMiller,
                                                                linkIntensity,
                                                                data_x,
                                                                data_y,
                                                                matrix,
                                                                paramDet,
                                                                boolctrl,
                                                                use_weights=use_weights,
                                                                dim=(2048, 2048),
                                                                pixelsize=pixelsize,
                                                                addoutput=1)

        # spotexpnum_spotsimnum
        # print linkedspots
        # spotexpnum_hkl
        # print linkExpMiller
        # intensitylist
        # print linkIntensity
        if dr != None:
            pixdev = dr[-2]
            if addoutput == 0:
                return newparam, newmat, pixdev, nb_of_pairs
            else:
                return newparam, newmat, pixdev, nb_of_pairs, pixdevlist, linkExpMiller
        else:
            if addoutput == 0:
                return None, None, None, None
            else:
                return None, None, None, None, None, None


def decode_params(string):
    """
    translate string made of 0 1 to resp False True
    """
    b = [True] * len(string)
    for k, s in enumerate(string):
        if s == "0":
            b[k] = False
    return b


def test_calibration_refinement():
    """
    illustrate how to use calibration refinement procedure.

    This converges if initial guess are quite very close to the solution
    (misorientation and unknown detector parameter lead straight to a wrong spots association and then to a wrong )
    """
    dirname = "./Examples/Ge/"
    imagefilename = "Ge0001.mccd"

    # must find
    # defaultParam = [69.190641489891689, 1050.503353382659, 1115.585040635892, 0.14566218092113614, -0.24339580573683331]
    defaultParam = [69.2, 1050.0, 1115, 0, 0]
    element = "Ge"

    pixelsize = 165.0 / 2048
    # dim = (2048, 2048)

    # Automation of peaksearch + indexation + calibration refinement

    # peak search
    time_0 = ttt.time()

    prefix = "autocalibtest_Ge"
    Isorted, _, _ = rmccd.PeakSearch(os.path.join(dirname, imagefilename),
                                                    PixelNearRadius=5,
                                                    removeedge=2,
                                                    IntensityThreshold=500,
                                                    boxsize=5,
                                                    position_definition=1,
                                                    verbose=1,
                                                    fit_peaks_gaussian=1,
                                                    xtol=0.001,
                                                    return_histo=0,
                                                    FitPixelDev=2.0)

    filewritten = RWASCII.writefile_Peaklist(prefix,
                                            Isorted,
                                            position_definition=1,
                                            overwrite=1,
                                            initialfilename=imagefilename,
                                            comments="",
                                            dirname=dirname)  #'.')

    if filewritten:
        print("peak list written in %s" % (prefix + ".dat"))
        print("Working directory is now", dirname)
        print("execution time: %.2f sec" % (ttt.time() - time_0))
    else:
        raise ValueError("Please Change directory name")

    dat_filename = prefix + ".dat"

    print("current directory", os.curdir)

    twicetheta, chi, dataintensity, data_x, data_y = F2TC.Compute_data2thetachi(dat_filename,
                                                                            (0, 1, 3),
                                                                            0,
                                                                            sorting_intensity="yes",
                                                                            param=defaultParam,
                                                                            pixelsize=pixelsize)
    # file_peaks = prefix + ".dat"
    # data = (twicetheta, chi, dataintensity, file_peaks)
    # Data_X, Data_Y, Data_I, File_NAME = data # 2theta and chi are now meant as X and Y
    # Data_index_expspot = np.arange(len(Data_X))
    data_exp = (twicetheta, chi, dataintensity, data_x, data_y)

    # starting matrix
    # UB or almost orientation matrix
    startingmatrix = [[0.092608021691080941, -0.97293680654779968, -0.21169793293699135],
                    [0.58953181179698111, 0.22490742744822317, -0.77578363483006851],
                    [0.80239914976161164, -0.052985258144157166, 0.59445649061416561]]

    # add some noise
    # matrot = GT.matRot(np.array([1,1,1]),0.5)
    # startingmatrix = np.dot(startingmatrix,matrot)

    emax = 15
    veryclose_angletol = 1

    newparam, newmat, pixDev, nb_of_pairs = GetCalibParameter(emax,
                                                            veryclose_angletol,
                                                            element,
                                                            startingmatrix,
                                                            defaultParam,
                                                            data_exp,
                                                            pixelsize=165.0 / 2048,
                                                            dim=(2048, 2048),
                                                            boolctrl="1" * 8)

    print("\n**************************************")
    print("pixDev :", pixDev)
    print("nb of peaks : ", nb_of_pairs)
    print("**************************************\n")

    print("\n****************************\n--------------------------2ND Fit ----------------------------\n")

    emax = 18
    veryclose_angletol = 0.5

    # Update
    defaultParam = newparam[:5]
    startingmatrix = newmat

    newparam, newmat, pixDev, nb_of_pairs = GetCalibParameter(emax,
                                                            veryclose_angletol,
                                                            element,
                                                            startingmatrix,
                                                            defaultParam,
                                                            data_exp,
                                                            pixelsize=165.0 / 2048,
                                                            dim=(2048, 2048))

    print("\n**************************************")
    print("pixDev :", pixDev)
    print("nb of peaks : ", nb_of_pairs)
    print("**************************************\n")


def test_SpotLinks():

    twicetheta_exp = np.array([90, 140, 50])
    chi_exp = np.array([70, 45, 85])
    dataintensity_exp = np.array([2400, 2, 2])
    veryclose_angletol = 1

    twicetheta = np.array([90.5, 140.5, -50, 90.5])
    chi = np.array([70, 45, 85, 70.4])
    Miller_ind = np.array([[8, 9, 10], [-5, 6, 0], [3, 2, 1], [6, 5, -5]])

    # posx = np.array([1024, 1000, 2000, 2000.9])
    # posy = np.array([200.6, 600.8, 800.9, 900.9])
    energy = np.array([10.12345, 11.21454796, 12.3179452, 14.547962])

    res = ISS.SpotLinks(twicetheta_exp,
                            chi_exp,
                            dataintensity_exp,  # experimental data
                            veryclose_angletol,  # tolerance angle
                            twicetheta,
                            chi,
                            Miller_ind,
                            energy)  # theoretical data

    if res:
        refine_indexed_spots, linkedspots_link, linkExpMiller_link, linkIntensity_link, linkResidues_link, Energy_Exp_spot, _ = res

    print("refine_indexed_spots", refine_indexed_spots)
    print("linkedspots_link", linkedspots_link)
    print("linkExpMiller_link", linkExpMiller_link)
    print("linkIntensity_link", linkIntensity_link)
    print("linkResidues_link", linkResidues_link)
    print("Energy_Exp_spot", Energy_Exp_spot)


# --- -----------   MAIN


if __name__ == "__main__":

    # test_calibration_refinement()

    # test simulation of strain and then strain retrieval from strain refinement of data

    angsample = 40
    ang = angsample * np.pi / 180.0
    # Ms = np.array([[np.cos(ang),0,np.sin(ang)],[0,1,0],[-np.sin(ang),0 ,np.cos(ang)]])
    Ms = GT.matRot(np.array([0, 1, 0]), 40)
    Msinv = GT.matRot(np.array([0, 1, 0]), -40)

    dirname = "."
    elem_label = "Cu"
    latparam = 3.6

    defaultParam = [69.66221, 895.29492, 960.78674, 0.84324, -0.32201]
    kf_direction = "Z>0"
    emax = 22.0
    emin = 5.0

    pixelsize = 165.0 / 2048

    ## Cu with orientsurf111 stretched by 1 % in bstar direction
    # DataPlot_filename = 'Cu1.cor'

    DataPlot_filename = "pipo.cor"
    B = 1 / latparam * np.array([[1.0, 0, 0], [0, 1.0, 0], [0, 0, 1.01]])
    Extinc = "fcc"
    Ta = np.eye(3)
    Tc = np.eye(3)
    U = GT.matRot(np.array([1, -2, 0]), 0.0000)

    # if B is expressed in sample frame (x//ki but tilted by 40deg z top, y towards the door
    B1 = B
    B = np.dot(Msinv, B1)
    # U = np.eye(3)

    # BUILDING fake DATA:

    # key_material, Extinc, Ta, U, B, Tc = Laue_classic_param # from combos
    # q = Ta U B Tc G*

    GrainSimulParam = [0, 0, 0, 0]
    # print "\n**************"
    # print "using Bmatrix containing lattice parameter"
    # print "****************\n"

    # take then parameters from combos
    GrainSimulParam[0] = B
    GrainSimulParam[1] = Extinc
    GrainSimulParam[2] = np.dot(np.dot(Ta, U), Tc)
    GrainSimulParam[3] = "inputB"

    spots2pi = LAUE.getLaueSpots(CST_ENERGYKEV / emax,
                                CST_ENERGYKEV / emin,
                                [GrainSimulParam],
                                fastcompute=0,
                                verbose=0,
                                kf_direction=kf_direction)

    # 2theta, chi of spot which are on camera (with harmonics)
    TwicethetaChi = LAUE.filterLaueSpots(spots2pi,
                                        fileOK=0,
                                        fastcompute=0,
                                        detectordistance=defaultParam[0],  # TODO: check if it is really the parameter set we need
                                        detectordiameter=165 * 1.2,
                                        kf_direction="Z>0",
                                        HarmonicsRemoval=1,
                                        pixelsize=pixelsize)

    Twicetheta = [spot.Twicetheta for spot in TwicethetaChi[0]]
    Chi = [spot.Chi for spot in TwicethetaChi[0]]
    Miller_ind = [list(spot.Millers) for spot in TwicethetaChi[0]]
    Energy = [spot.EwaldRadius * CST_ENERGYKEV for spot in TwicethetaChi[0]]

    posx, posy = F2TC.calc_xycam_from2thetachi(Twicetheta,
                                                Chi,
                                                defaultParam,
                                                verbose=0,
                                                pixelsize=pixelsize)[:2]

    # READING COR file AND INDEXATION ----------------------------------------------------------------------------
    file_extension = "cor"

    data_theta = np.array(Twicetheta) / 2.0
    data_chi = np.array(Chi)
    data_pixX = np.array(posx)
    data_pixY = np.array(posy)
    data_I = np.ones(len(data_theta)) * 1000
    # filename = DataPlot_filename

    index_foundgrain = 0
    # dictionary of exp spots
    indexed_spots = {}
    for k in range(len(data_theta)):
        indexed_spots[k] = [k,  # index of experimental spot in .cor file
                            data_theta[k] * 2.0,
                            data_chi[k],  # 2theta, chi coordinates
                            data_pixX[k],
                            data_pixY[k],  # pixel coordinates
                            data_I[k],  # intensity
                            0]  # 0 means non indexed yet
    last_orientmatrix_fromindexation = {}
    last_Bmatrix_fromindexation = {}
    last_epsil_fromindexation = {}

    # Running indexation

    # updated exp spot index to be still index
    current_exp_spot_index_list = Non_Indexed_Spots(indexed_spots, len(data_theta))

    # for each grain or indexation step: ******************************
    # ******************************************************************

    # compute angular distance between all exp; spots:

    nbspots_in_data = len(data_theta[current_exp_spot_index_list])

    # Matching spots set Size (MSSS):
    nbspotmaxformatching = nbspots_in_data

    # select 1rstly spots that have not been indexed and 2ndly reduced list by user
    index_to_select = np.take(current_exp_spot_index_list, np.arange(nbspotmaxformatching))

    select_theta = data_theta[index_to_select]
    select_chi = data_chi[index_to_select]
    select_I = data_I[index_to_select]
    # print select_theta
    # print select_chi
    listcouple = np.transpose(np.array([select_theta, select_chi]))
    Tabledistance = GT.calculdist_from_thetachi(listcouple, listcouple)

    # classical indexation parameters:
    data = (2 * select_theta, select_chi, select_I, DataPlot_filename)

    spot_index_central = [4, 5, 6, 7]  # central spot or list of spots
    # spot_index_central = 0                  # central spot or list of spots
    nbmax_probed = 10  # 'Recognition spots set Size (RSSS): '

    energy_max = 22.0
    n = 3  # for LUT

    rough_tolangle = 0.5  # 'Dist. Recogn. Tol. Angle (deg)'
    fine_tolangle = 0.2  # 'Matching Tolerance Angle (deg)'
    Nb_criterium = 2  # 'Minimum Number Matched Spots: '
    print("rough_tolangle ", rough_tolangle)
    print("fine_tolangle ", fine_tolangle)
    NBRP = 5  # number of best result for each element of spot_index_central

    latticeparams = DictLT.dict_Materials[elem_label][1]
    B = CP.calc_B_RR(latticeparams)

    # indexation procedure
    bestmat, stats_res = INDEX.getOrientMatrices(spot_index_central,
                                                energy_max,
                                                Tabledistance[:nbmax_probed, :nbmax_probed],
                                                select_theta,
                                                select_chi,
                                                n,
                                                B,
                                                rough_tolangle=rough_tolangle,
                                                fine_tolangle=fine_tolangle,
                                                Nb_criterium=Nb_criterium,
                                                plot=0,
                                                structure_label=elem_label,
                                                nbbestplot=NBRP)

    # bestmat contains: NBRP * len(spot_index_central) matrix

    # Look for the very best matrix candidates: -----to be improved in readability ------------------------
    ar = []
    for elem in stats_res:
        ar.append([elem[0], -elem[2]])

    tabstat = np.array(ar)  # , dtype = [('x', '<i4'), ('y', '<i4')])
    rankmat = np.argsort(tabstat[:, 0])[::-1]  # ,order=('x','y'))

    verybestmat = bestmat[rankmat[0]]
    # -----------------------------------------------------------------------------------------------------
    # very best matrix
    orientmatrix = verybestmat

    # end of READING COR file AND INDEXATION ----------------------------------------------------------------------------

    # STRAIN REFINEMENT -----------------------------------------------
    # the resimulate to output miller indices
    emax_simul = 20
    veryclose_angletol = 0.2

    res_strain = GetStrainOrient(orientmatrix,
                                    B,
                                    elem_label,
                                    emax_simul,
                                    veryclose_angletol,
                                    data,
                                    detectorparameters=defaultParam,
                                    addoutput=0)

    # print "res_strain",res_strain

    newUBmat, newUmat, newBmat, deviatoricstrain, RR, linkResidues_fit, pixdev, nb_fitted_peaks, linkExpMiller_link = res_strain

    latticeparams = DictLT.dict_Materials[elem_label][1]
    deviatoric_strain, latticeparameterstrained = CP.DeviatoricStrain_LatticeParams(
        newUBmat, latticeparams)

    print("\n\ndeviatoric_strain", deviatoric_strain)
    print("latticeparameterstrained", latticeparameterstrained)
    print("newUmat", newUmat)

    def giveeps(mat):
        eps1 = np.dot(Ms, np.dot(mat, Msinv))
        eps2 = np.dot(Msinv, np.dot(mat, Ms))
        return eps1, eps2

    print("\n\nstrain sample frame", giveeps(deviatoricstrain))

    if 0:
        # ----------------------------------------------------------------------------
        # example with Ti
        dirname = "./Examples/Ti/"

        # Ti with orientsurf111 stretched by 1 % in bstar direction
        DataPlot_filename = "Ti_111_0p01_bstar.cor"
        elem_label = "Ti"

        # Ti_s with orientsurf111 lattice param 3 3 4.7 90.5 89.5 120.5
        DataPlot_filename = "Tis.cor"
        elem_label = "Ti"
        # Bmatrix = [[0.3868688469461854, 0.19633627174252782, -0.0010612910304965497],
        # [0.0, 0.33334602612857905, 0.0018567803810125216],
        # [0.0, 0.0, 0.21276595744680851]]

        defaultParam = [69.66221, 895.29492, 960.78674, 0.84324, -0.32201]

        pixelsize = 165.0 / 2048

        # Automation of peaksearch + indexation + strain refinement

        # peak search
        time_0 = ttt.time()

        # a C_#########.cor file is now created
        file_extension = "cor"

        # Reading . cor file
        # TODO use better readcorfile in readmccd.py
        #    Current_peak_data = scipy.io.array_import.read_array(os.path.join(dirname, DataPlot_filename), lines=(1, -1))
        Current_peak_data = np.loadtxt(os.path.join(dirname, DataPlot_filename), skiprows=1)

        # nbcolumns == 5:
        data_theta = Current_peak_data[:, 0] / 2.0
        data_chi, data_pixX, data_pixY, data_I = np.transpose(Current_peak_data)[1:]

        # filename = DataPlot_filename

        index_foundgrain = 0
        # dictionary of exp spots
        indexed_spots = {}
        for k in range(len(data_theta)):
            indexed_spots[k] = [
                k,  # index of experimental spot in .cor file
                data_theta[k] * 2.0,
                data_chi[k],  # 2theta, chi coordinates
                data_pixX[k],
                data_pixY[k],  # pixel coordinates
                data_I[k],  # intensity
                0]  # 0 means non indexed yet
        last_orientmatrix_fromindexation = {}
        last_Bmatrix_fromindexation = {}
        last_epsil_fromindexation = {}

        # Running indexation

        # updated exp spot index to be still index
        current_exp_spot_index_list = Non_Indexed_Spots(indexed_spots, len(data_theta))

        # for each grain or indexation step: ******************************
        # ******************************************************************

        # compute angular distance between all exp; spots:

        nbspots_in_data = len(data_theta[current_exp_spot_index_list])

        # Matching spots set Size (MSSS):
        nbspotmaxformatching = nbspots_in_data

        # select 1rstly spots that have not been indexed and 2ndly reduced list by user
        index_to_select = np.take(current_exp_spot_index_list, np.arange(nbspotmaxformatching))

        select_theta = data_theta[index_to_select]
        select_chi = data_chi[index_to_select]
        select_I = data_I[index_to_select]
        # print select_theta
        # print select_chi
        listcouple = np.transpose(np.array([select_theta, select_chi]))
        Tabledistance = GT.calculdist_from_thetachi(listcouple, listcouple)

        # classical indexation parameters:
        data = (2 * select_theta, select_chi, select_I, DataPlot_filename)

        spot_index_central = [0, 1, 2, 3]  # central spot or list of spots
        spot_index_central = 0  # central spot or list of spots
        nbmax_probed = 10  # 'Recognition spots set Size (RSSS): '

        energy_max = 20.0
        n = 3  # for LUT

        rough_tolangle = 0.5  # 'Dist. Recogn. Tol. Angle (deg)'
        fine_tolangle = 0.2  # 'Matching Tolerance Angle (deg)'
        Nb_criterium = 15  # 'Minimum Number Matched Spots: '
        print("rough_tolangle ", rough_tolangle)
        print("fine_tolangle ", fine_tolangle)
        NBRP = 1  # number of best result for each element of spot_index_central

        latticeparams = DictLT.dict_Materials[elem_label][1]
        B = CP.calc_B_RR(latticeparams)

        # indexation procedure
        bestmat, stats_res = INDEX.getOrientMatrices(spot_index_central,
                                                    energy_max,
                                                    Tabledistance[:nbmax_probed, :nbmax_probed],
                                                    select_theta,
                                                    select_chi,
                                                    n,
                                                    B,
                                                    rough_tolangle=rough_tolangle,
                                                    fine_tolangle=fine_tolangle,
                                                    Nb_criterium=Nb_criterium,
                                                    plot=0,
                                                    structure_label=elem_label,
                                                    nbbestplot=NBRP)

        # bestmat contains: NBRP * len(spot_index_central) matrix

        # Look for the very best matrix candidates: -----to be improved in readability ------------------------
        ar = []
        for elem in stats_res:
            ar.append([elem[0], -elem[2]])

        tabstat = np.array(ar)  # , dtype = [('x', '<i4'), ('y', '<i4')])
        rankmat = np.argsort(tabstat[:, 0])[::-1]  # ,order=('x','y'))

        verybestmat = bestmat[rankmat[0]]
        # -----------------------------------------------------------------------------------------------------
        # very best matrix
        orientmatrix = verybestmat

        # the resimulate to output miller indices
        emax_simul = 16
        veryclose_angletol = 0.2

        res_strain = GetStrainOrient(orientmatrix,
                                        B,
                                        elem_label,
                                        emax_simul,
                                        veryclose_angletol,
                                        data,
                                        detectorparameters=defaultParam,
                                        addoutput=0)

        (newUBmat, newUmat, newBmat, deviatoricstrain, RR,
        linkResidues_fit, pixdev, nb_fitted_peaks) = res_strain

        latticeparams = DictLT.dict_Materials[elem_label][1]
        deviatoric_strain, latticeparameterstrained = CP.DeviatoricStrain_LatticeParams(
            newUBmat, latticeparams)



"""
GUI Module of LaueTools package for choosing Detector File Parameters
"""
import sys

import wx

if sys.version_info.major == 3:
    from .. import dict_LaueTools as DictLT
    from .. import generaltools as GT
else:
    import dict_LaueTools as DictLT
    import generaltools as GT

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN
    wx.CHANGE_DIR = wx.FD_CHANGE_DIR

    def sttip(argself, strtip):
        return wx.Window.SetToolTip(argself, wx.ToolTip(strtip))

    wx.Window.SetToolTipString = sttip


# --- ---------------  Binary image File parameters
class CCDFileParameters(wx.Dialog):
    """
    Class to set binary image file parameters
    """
    def __init__(self, parent, _id, title, CCDlabel):
        """
        initialize board window
        """
        wx.Dialog.__init__(self, parent, _id, title, size=(800, 600), style=wx.RESIZE_BORDER)
        self.parent = parent
        #print("self.parent", self.parent)

        txt = wx.StaticText(self, -1, "Choose readout parameters of CCD image file")
        font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        txt.SetFont(font)
        txt.SetForegroundColour((255, 0, 0))

        dict_CCD = DictLT.dict_CCD
        self.allCCD_names = list(dict_CCD.keys())
        self.allCCD_names.sort()

        font3 = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)
        self.paramdetector = ["CCD label",
                            "header offset",
                            "dataformat",
                            "fliprot",
                            "saturation Value",
                            "file extension"]
        self.units = ["ascii",
                        "byte",
                        "python format",
                        "operator tag",
                        "integer",
                        "ascii"]

        self.CCDLabel = CCDlabel
        self.readCCDparams()

        self.newparam = []

        self.controltext = []

        a0 = wx.StaticText(self, -1, "CCD Image File type")
        a0.SetFont(font3)

        self.allCCD_names = GT.put_on_top_list(("MARCCD165",
                                            "sCMOS",
                                            "----------",
                                            "PRINCETON",
                                            "VHR_Mar13",
                                            "----------"),
                                            self.allCCD_names,
                                            forceinsertion=True)

        self.comboCCD = wx.ComboBox(self,
                                    -1,
                                    self.CCDLabel,
                                    size=(180, -1),
                                    choices=self.allCCD_names,
                                    style=wx.CB_READONLY)

        self.comboCCD.Bind(wx.EVT_COMBOBOX, self.EnterComboCCD)

        a1 = wx.StaticText(self, -1, "parameter")
        a2 = wx.StaticText(self, -1, "current value")
        a3 = wx.StaticText(self, -1, "initial value")
        a4 = wx.StaticText(self, -1, "unit")

        for text in [a1, a2, a3, a4]:
            text.SetFont(font3)

        com = wx.StaticText(self, -1, "Comments")
        com.SetFont(font3)

        self.comments = wx.TextCtrl(self, style=wx.TE_MULTILINE, size=(580, 50))
        self.comments.SetValue(DictLT.dict_CCD[self.CCDLabel][-1])

        self.DisplayValues(1)

        btnaccept = wx.Button(self, -1, "Accept", (300, 40))
        btnaccept.Bind(wx.EVT_BUTTON, self.OnAccept)
        btnaccept.SetDefault()

        btncancel = wx.Button(self, -1, "Cancel", (150, 40))
        btncancel.Bind(wx.EVT_BUTTON, self.OnCancel)

        self.keepon = False

        if WXPYTHON4:
            grid = wx.FlexGridSizer(4, 7, 10)
        else:
            grid = wx.FlexGridSizer(4, 7)

        grid.Add(a1)
        grid.Add(a2)
        grid.Add(a3)
        grid.Add(a4)
        for krow in range(6):
            grid.Add(wx.StaticText(self, -1, self.paramdetector[krow]))
            grid.Add(self.controltext[krow])
            grid.Add(wx.StaticText(self, -1, str(self.value[krow])))
            grid.Add(wx.StaticText(self, -1, self.units[krow]))

        hboxcombo = wx.BoxSizer(wx.HORIZONTAL)
        hboxcombo.Add(a0)
        hboxcombo.Add(self.comboCCD)

        hboxbtn = wx.BoxSizer(wx.HORIZONTAL)
        hboxbtn.Add(btnaccept)
        hboxbtn.Add(btncancel)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txt)
        vbox.Add(hboxcombo)
        vbox.Add(grid)
        vbox.Add(com)
        vbox.Add(self.comments)
        vbox.Add(hboxbtn)

        self.SetSizer(vbox)

        # tooltip
        btnaccept.SetToolTipString("Accept")

    def readCCDparams(self):
        """ read CCD parameters from self.CCDLabel
        
        set self.value"""
        CCDparams = DictLT.dict_CCD[str(self.CCDLabel)]

        # TODO add framedim and pixelsize as CCD parameters better than as Detector parameters
        (self.parent.framedim,
            self.parent.pixelsize,
            self.saturationvalue,
            self.fliprot,
            self.headeroffset,
            self.dataformat,
            self.commentstxt,
            self.file_extension) = CCDparams

        self.value = (self.CCDLabel,
                    self.headeroffset,
                    self.dataformat,
                    self.fliprot,
                    self.saturationvalue,
                    self.file_extension)
        #print("self.value", self.value)


    def DisplayValues(self, _):
        """
        display and set values of parameters

        set self.controltext list
        """
        if self.controltext == []:
            for kk, _ in enumerate(self.paramdetector):
                self.controltext.append(wx.TextCtrl(self, -1, str(self.value[kk])))
        else:
            for kk, _ in enumerate(self.paramdetector):
                self.controltext[kk].SetValue(str(self.value[kk]))
    def DeleteValues(self, _):
        """
        set parameters to zero or None
        """
        for kk, _ in enumerate(self.paramdetector):
            self.controltext.append(wx.TextCtrl(self, -1, str(None)))

    def EnterComboCCD(self, event):
        """ select detector item """
        item = event.GetSelection()
        self.CCDLabel = self.allCCD_names[item]
        print("item", item)
        print("CCDLabel", self.CCDLabel)

        if self.CCDLabel.startswith("--"):
            self.DeleteValues(event)
            self.comments.SetValue("")
            return

        self.readCCDparams()

        #print("self.value", self.value)
        self.comments.SetValue(self.commentstxt)
        self.DisplayValues(event)
        event.Skip()

    def OnAccept(self, _):
        """ accept current parameters and set self.parent attributes:
        headeroffset, dataformat, fliprot, saturationvalue, CCDLabel, file_extension
        framedim, pixelsize, detectordiameter """
        if not self.CCDLabel.startswith("--"):
            # file parameters
            self.parent.headeroffset = self.headeroffset
            self.parent.dataformat = self.dataformat
            self.parent.fliprot = self.fliprot
            self.parent.saturationvalue = self.saturationvalue
            self.parent.CCDLabel = self.CCDLabel
            self.parent.file_extension = self.file_extension
            # geometrical parameters
            self.parent.framedim, self.parent.pixelsize = DictLT.dict_CCD[str(self.CCDLabel)][0:2]

            #print("self.parent", self.parent)
            print("self.parent.framedim", self.parent.framedim)
            print("self.parent.pixelsize", self.parent.pixelsize)

            self.parent.detectordiameter = (max(self.parent.framedim) * self.parent.pixelsize)
            self.Close()
        else:
            wx.MessageBox("Please select on type of CCD camera or Cancel", "Error")

    def OnCancel(self, _):
        self.Close()



import os
import sys

import numpy as np

import wx
import wx.grid

if sys.version_info.major == 3:
    from .. import IOLaueTools as IOLT
else:
    import IOLaueTools as IOLT

# --- ---------------  Plot limits board  parameters
# class PeaksListBoard(wx.Dialog):
class PeaksListBoard(wx.Frame):
    """
    Class to select scatter plot properties of peak list
    """
    def __init__(self, parent, _id):
        """
        initialize board window
        """
        # - Initialize the window:
        wx.Frame.__init__(self, None, wx.ID_ANY, "Peak list plot properties", size=(600, 400))

        # Add a panel so it looks correct on all platforms
        self.panel = wx.Panel(self, wx.ID_ANY)

        self.parent = parent
        #print("self.parent", self.parent)
        self.selectedMarker = None

        self.list_markerstyle = ["+", "x", "o", "h", "*", "p"]

        self.scatterplot_list = []

        self.nb_peakslists = 0
        self.myDataList = []

        self.selectedcolor = None
        self.selectedpeaklist = None
        self.fullpathfilename = None
        self.myscatter = None

        # widgets ------------------------------
        txtimage = wx.StaticText(self.panel, -1, "Image file path:")

        self.expimagetxtctrl = wx.TextCtrl(self.panel, -1, "", size=(400, -1))
        self.expimagebrowsebtn = wx.Button(self.panel, -1, "...", size=(50, -1))

        self.expimagebrowsebtn.Bind(wx.EVT_BUTTON, self.onSelectPeaksListFile)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(txtimage, 0, wx.EXPAND)
        hsizer.Add(self.expimagetxtctrl, 0, wx.EXPAND)
        hsizer.Add(self.expimagebrowsebtn, 0, wx.EXPAND)

        num_of_rows = 1
        num_of_columns = 5
        self.grid = wx.FlexGridSizer(num_of_rows, num_of_columns, 0, 0)

        self.grid.Add(wx.StaticText(self.panel, -1, "Select"))
        self.grid.Add(wx.StaticText(self.panel, -1, "Name"))
        self.grid.Add(wx.StaticText(self.panel, -1, "Color"))
        self.grid.Add(wx.StaticText(self.panel, -1, "marker"))
        self.grid.Add(wx.StaticText(self.panel, -1, "size"))

        #         toolbar = self.CreateToolBar()
        #         qtool = toolbar.AddLabelTool(wx.ID_ANY, 'Refresh',
        #                                      wx.Bitmap(os.path.join(os.path.dirname(__file__),
        #                                                             'refresh.png')))
        #         toolbar.AddSeparator()
        #         self.Bind(wx.EVT_TOOL, self.refreshButton, qtool)
        #         toolbar.Realize()

        self.vsizer = wx.BoxSizer(wx.VERTICAL)
        self.vsizer.Add(hsizer, 0, wx.ALL)
        self.vsizer.Add(self.grid, 0, wx.EXPAND)

        self.panel.SetSizer(self.vsizer)
        #         self.Fit()
        self.vsizer.Layout()

    def onChangeColor(self, event, btnlabel):
        """
        """
        print("event", event)
        index = int(btnlabel.split("_")[-1])
        print("clicked btn index", index)
        dlg = wx.ColourDialog(self)

        # Ensure the full colour dialog is displayed,
        # not the abbreviated version.
        dlg.GetColourData().SetChooseFull(True)

        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetColourData()

            self.selectedcolor = data.GetColour().Get()
            print("You selected: ", self.selectedcolor)
            print("You selected: (%d, %d, %d)\n" % self.selectedcolor[:3])
            print("matplolib color: (%.2f, %.2f, %.2f)\n" % self.convert_to_rgbmatplotlib(self.selectedcolor[:3]))

            self.myDataList[index]["Color"] = self.selectedcolor

            # update color btn
            clickedbtn = getattr(self, "colorbtn_%d" % index)
            clickedbtn.SetBackgroundColour(wx.Colour(*self.selectedcolor))

        dlg.Destroy()

        if self.scatterplot_list is not []:
            self.scatterplot_list[index].set_edgecolor(self.convert_to_rgbmatplotlib(self.selectedcolor))
            self.parent.update_draw(1)

    def convert_to_rgbmatplotlib(self, rgbcolor):
        """convert rgb to rgbmatplotlib
        """
        return (rgbcolor[0] / 255.0, rgbcolor[1] / 255.0, rgbcolor[2] / 255.0)

    def refreshButton(self, _):
        """Refresh
        """
        self.refreshMyData()
        self.refreshGrid()

    def refreshwholeGrid(self):
        print("\n\n\n------------refresh grid")
        # - Clear the grid:
        print("Size of myDataList: ", len(self.myDataList))

        num_of_rows = len(self.myDataList) + 1

        self.grid.SetRows(num_of_rows)

        self.grid.Add(wx.StaticText(self.panel, -1, "Select"))
        self.grid.Add(wx.StaticText(self.panel, -1, "Name"))
        self.grid.Add(wx.StaticText(self.panel, -1, "Color"))
        self.grid.Add(wx.StaticText(self.panel, -1, "marker"))
        self.grid.Add(wx.StaticText(self.panel, -1, "size"))

        # - Populate the grid with new data:
        for i in range(len(self.myDataList)):
            colorbtn = wx.Button(self.panel, -1, "color_%d" % i)
            colorbtn.SetBackgroundColour(wx.Colour(*self.myDataList[i]["Color"]))
            setattr(self, "colorbtn_%d" % i, colorbtn)
            colorbtn.Bind(wx.EVT_BUTTON,
                lambda evt, name=colorbtn.GetLabel(): self.onChangeColor(evt, name))

            markercombo = wx.ComboBox(self.panel,
                                        -1,
                                        str(self.myDataList[i]["marker"]),
                                        choices=self.list_markerstyle,
                                        size=(20, -1))
            #             setattr(self, 'markercombo_%d' % i, markercombo)
            markercombo.Bind(wx.EVT_COMBOBOX,
                                lambda evt, comboindex=i: self.onChangeMarker(evt, comboindex))

            self.grid.Add(wx.StaticText(self.panel, -1, str(self.myDataList[i]["Select"])))
            self.grid.Add(wx.StaticText(self.panel, -1, str(self.myDataList[i]["Name"])))
            self.grid.Add(colorbtn)
            self.grid.Add(markercombo)
            self.grid.Add(wx.StaticText(self.panel, -1, str(self.myDataList[i]["size"])))

        self.panel.SetSizer(self.vsizer)
        self.vsizer.Layout()

    def addRow_Grid(self, datalist_index):
        print("\n\n\n------------refresh grid")
        # - Clear the grid:
        print("Size of myDataList: ", len(self.myDataList))

        num_of_rows = self.nb_peakslists + 1

        self.grid.SetRows(num_of_rows)

        # - Populate the grid with new data:
        chckbox = wx.CheckBox(self.panel, -1)
        chckbox.SetValue(True)
        chckbox.Bind(wx.EVT_CHECKBOX,
            lambda evt, chckindex=datalist_index: self.onCheckBox(evt, chckindex))
        setattr(self, "chckbox_%d" % datalist_index, chckbox)

        txtctrlname = wx.TextCtrl(self.panel, -1, str(self.myDataList[datalist_index]["Name"]))
        txtctrlname.SetMinSize((300, -1))
        #         setattr(self, 'namectrl_%d' % datalist_index, txtctrlname)

        colorbtn = wx.Button(self.panel, -1, "color_%d" % datalist_index)
        colorbtn.SetBackgroundColour(wx.Colour(*self.myDataList[datalist_index]["Color"]))
        setattr(self, "colorbtn_%d" % datalist_index, colorbtn)
        colorbtn.Bind(wx.EVT_BUTTON,
            lambda evt, name=colorbtn.GetLabel(): self.onChangeColor(evt, name))

        markercombo = wx.ComboBox(self.panel,
                                    -1,
                                    str(self.myDataList[datalist_index]["marker"]),
                                    choices=self.list_markerstyle,
                                    size=(80, -1))
        #             setattr(self, 'markercombo_%d' % datalist_index, markercombo)
        markercombo.Bind(wx.EVT_COMBOBOX,
            lambda evt, comboindex=datalist_index: self.onChangeMarker(evt, comboindex))

        sizectrl = wx.SpinCtrl(self.panel, -1, str(self.myDataList[datalist_index]["size"]))
        setattr(self, "sizectrl_%d" % datalist_index, sizectrl)
        sizectrl.Bind(wx.EVT_SPINCTRL,
            lambda evt, index=datalist_index: self.onChangeSize(evt, index))

        self.grid.Add(chckbox, 1)
        self.grid.Add(txtctrlname, 1)
        self.grid.Add(colorbtn, 1)
        self.grid.Add(markercombo, 1)
        self.grid.Add(sizectrl, 1)

        self.grid.AddGrowableCol(2, 1)

        self.panel.SetSizer(self.vsizer)
        self.vsizer.Layout()

    def onCheckBox(self, _, chckindex):
        print("chckindex", chckindex)

        chckbox = getattr(self, "chckbox_%d" % chckindex)

        newstate = chckbox.GetValue()
        print("newstate", newstate)

        if not newstate:
            self.scatterplot_list[chckindex].remove()
            self.parent.update_draw(1)
        else:
            self.plotmarkers(addscatteratindex=chckindex,
                            markerstyle=self.myDataList[chckindex]["marker"],
                            edgecolor=self.convert_to_rgbmatplotlib(
                                self.myDataList[chckindex]["Color"]))

    def onChangeSize(self, _, index):
        """ on change size
        """
        sizectrl = getattr(self, "sizectrl_%d" % index)

        newsize = int(sizectrl.GetValue())
        corresponding_scatterplot = self.scatterplot_list[index]

        print(dir(corresponding_scatterplot))

        if self.scatterplot_list is not []:
            self.scatterplot_list[index].remove()
            self.plotmarkers(addscatteratindex=index,
                            markerstyle=self.myDataList[index]["marker"],
                            edgecolor=self.convert_to_rgbmatplotlib(
                                self.myDataList[index]["Color"]),
                            markersize=newsize)

            self.myDataList[index]["size"] = newsize

    def onChangeMarker(self, event, comboindex):
        """ on change marker type
        """
        print("combo index", comboindex)
        item = event.GetSelection()
        self.selectedMarker = self.list_markerstyle[item]

        self.myDataList[comboindex]["marker"] = self.selectedMarker

        print("select marker: %s\n" % self.selectedMarker)

        if self.scatterplot_list is not []:
            self.scatterplot_list[comboindex].remove()
            self.plotmarkers(addscatteratindex=comboindex,
                            markerstyle=self.selectedMarker,
                            edgecolor=self.convert_to_rgbmatplotlib(
                                self.myDataList[comboindex]["Color"]))

        event.Skip()

    def readnewpeaklistfile(self):

        fullpathfilename = str(self.expimagetxtctrl.GetValue())
        if not os.path.isfile(fullpathfilename):
            dlg = wx.MessageDialog(self,
                                "peak list file : %s\n\ndoes not exist!!" % fullpathfilename,
                                "error",
                                wx.OK | wx.ICON_ERROR)
            #                 dlg = wx.MessageDialog(self, 'Detector parameters must be float with dot separator',
            #                                    'Bad Input Parameters',)
            dlg.ShowModal()
            dlg.Destroy()
            return

        if fullpathfilename.endswith(".dat"):
            self.selectedpeaklist = IOLT.read_Peaklist(fullpathfilename)[:, :2]
        elif fullpathfilename.endswith(".cor"):
            pl = IOLT.readfile_cor(fullpathfilename)[3:5]
            self.selectedpeaklist = np.array(pl).T
        elif fullpathfilename.endswith(".fit"):
            res = IOLT.readfitfile_multigrains(fullpathfilename, return_columnheaders=True)
            try:
                data, colname_dict = res
            except:
                print("problem when reading .fit")
                print(res)

            col_X = None
            if "Xtheo" in colname_dict:
                col_X = colname_dict["Xtheo"]
                print("column for theo. peaks X position found at index %d" % col_X)
            elif "Xexp" in colname_dict:
                col_X = colname_dict["Xexp"]
                print("column for exp. peaks X position found at index %d" % col_X)
            else:
                print("\n\n!!column for theo. or exp. peaks X position not found...!!\n")

            col_Y = None
            if "Ytheo" in colname_dict:
                col_Y = colname_dict["Ytheo"]
                print("column for theo. peaks Y position found at index %d" % col_Y)
            elif "Yexp" in colname_dict:
                col_Y = colname_dict["Yexp"]
                print("column for exp. peaks Y position found at index %d" % col_Y)
            else:
                print("\n\n!!column for theo. or exp. peaks Y position not found...!!\n")

            if len(data) == 2:
                allspotsdata = data[0][4]
            else:
                allspotsdata = data[4]

            if col_X is None or col_Y is None:
                print("\n\n!!Can't read the file ...!!\n")
                return

            self.selectedpeaklist = np.take(allspotsdata, (col_X, col_Y), axis=1)

        self.fullpathfilename = fullpathfilename
        self.nb_peakslists += 1

        print("self.selectedpeaklist", self.selectedpeaklist)

    def update_ImageArray(self):
        self.parent.ImageArray = self.ImageArray

        self.updateplot()

    def onSelectPeaksListFile(self, evt):
        self.GetfullpathFile(evt)
        self.expimagetxtctrl.SetValue(self.fullpathfilename)

        self.readnewpeaklistfile()

        self.myDataList.append({"Select": True,
                                "Name": os.path.split(self.fullpathfilename)[-1],
                                "Color": (0, 255, 0),
                                "marker": "*",
                                "size": 20,
                                "peakslist": self.selectedpeaklist})
        #         self.update_ImageArray()
        self.addRow_Grid(self.nb_peakslists - 1)
        self.plotmarkers()

    def GetfullpathFile(self, _):
        wcd = "Peaks list (*.dat)|*.dat|Peaks list (*.cor)|*.cor|indexed Peaks list (*.fit)|*.fit|All files(*)|*"
        myFileDialog = wx.FileDialog(self, "Choose an image file", style=wx.OPEN, wildcard=wcd)
        dlg = myFileDialog
        dlg.SetMessage("Choose an image file")
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()

            self.fullpathfilename = str(filename)
        else:
            pass

    def plotmarkers(self, addscatteratindex=None, markerstyle="o", edgecolor="g", markersize=20):
        """add plot markers at peaks pixel position from self.selectedpeaklist or self.myDataList
        """
        # offset convention
        X_OFFSET = 1
        Y_OFFSET = 1

        kwords = {"marker": markerstyle, "facecolor": "None",
                    "edgecolor": edgecolor, "s": markersize}

        if addscatteratindex is not None:
            X, Y = self.myDataList[addscatteratindex]["peakslist"].T
        else:
            X, Y = self.selectedpeaklist.T

        self.myscatter = self.parent.axes.scatter(X - X_OFFSET, Y - Y_OFFSET, alpha=1.0, **kwords)

        if addscatteratindex is not None:
            self.scatterplot_list[addscatteratindex] = self.myscatter
        else:
            self.scatterplot_list.append(self.myscatter)

        #         print dir(self.myscatter)
        self.parent.update_draw(1)


if __name__ == "__main__":

    class App(wx.App):
        def OnInit(self):
            """Create the main window and insert the custom frame"""
            dlg = PeaksListBoard(None, -1)

            dlg.Show(True)
            return True

    app = App(0)
    app.MainLoop()



"""
Class to have interaction between GUI and a thread

The Frame class has got the GUI as parent. It starts a thread worker based on a function for a 
long task f(args, kwargs).
This function f must have in its optional keywords arguments the key 'worker' with value 
a worker set by the class.
In the function f, the worker attributes _want_abort, fctOutputResults 
and method callbackfct() can be called

_want_abort :  1   request to stop the thread
fctOutputResults :  output of function f , at the end of f: worker.fctOutputResults
must be set to the normal output of f
callbackfct: method to send message from the function to the GUI Frame class

see also example in
indexingAnglesLUT.getUBs_and_MatchingRate()
LaueToolsGUI.ManualIndexFrame.Reckon_2pts_new()
"""

import time
from threading import Thread
import wx

# Button definitions
ID_START = wx.NewId()
ID_STOP = wx.NewId()

# Define notification event for thread completion
EVT_RESULT_ID = wx.NewId()


def EVT_RESULT(win, func):
    """Define Result Event."""
    win.Connect(-1, -1, EVT_RESULT_ID, func)


class ResultEvent(wx.PyEvent):
    """Simple event to carry arbitrary result data."""

    def __init__(self, data):
        """Init Result Event."""
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_RESULT_ID)
        self.data = data


# Thread class that executes processing
class WorkerThread(Thread):
    """Worker Thread Class."""

    def __init__(self, notify_window, fctparams):
        """Init Worker Thread Class."""
        self.fctparams = fctparams
        self.fctOutputResults = None
        Thread.__init__(self)
        self._notify_window = notify_window
        self._want_abort = 0
        # This starts the thread running on creation, but you could
        # also make the GUI thread responsible for calling this
        self.start()

    #     def run_old(self):
    #         """Run Worker Thread."""
    #         # This is the code executing in the new thread. Simulation of
    #         # a long process (well, 10s here) as a simple loop - you will
    #         # need to structure your processing so that you periodically
    #         # peek at the abort variable
    #         for i in range(10):
    #             time.sleep(1)
    #             if self._want_abort:
    #                 # Use a result of None to acknowledge the abort (of
    #                 # course you can use whatever you'd like or even
    #                 # a separate event type)
    #                 wx.PostEvent(self._notify_window, ResultEvent(None))
    #                 return
    #         # Here's where the result would be returned (this is an
    #         # example fixed result of the number 10, but it could be
    #         # any Python object)
    #         wx.PostEvent(self._notify_window, ResultEvent(True))

    def callbackfct(self, arg):
        wx.PostEvent(self._notify_window, ResultEvent(arg))

    def run(self):
        """Run Worker Thread."""
        # This is the code executing in the new thread. Simulation of
        # a long process (well, 10s here) as a simple loop - you will
        # need to structure your processing so that you periodically
        # peek at the abort variable
        # INDEX.getUBs_and_MatchingRate #
        fct, args, keyargs = self.fctparams
        keyargs["worker"] = self
        fct(*args, **keyargs)

    #         for i in range(10):
    #             time.sleep(1)
    #             if self._want_abort:
    #                 # Use a result of None to acknowledge the abort (of
    #                 # course you can use whatever you'd like or even
    #                 # a separate event type)
    #                 wx.PostEvent(self._notify_window, ResultEvent(None))
    #                 return
    # Here's where the result would be returned (this is an
    # example fixed result of the number 10, but it could be
    # any Python object)

    def abort(self):
        """abort worker thread."""
        # Method for use by main thread to signal an abort
        self._want_abort = 1


# GUI Frame class that spins off the worker thread
class ThreadHandlingFrame(wx.Frame):
    """simple Frame Class handling a thread (worker) to started and aborted"""

    def __init__(self, parent, _id, threadFunctionParams=None,
                                parentAttributeName_Result=None,
                                parentNextFunction=None):
        """Create the MainFrame.
        @param threadFunctionParams: threadfunction parameters: list of function name, function arguments, function optional argument (dict)
        @param parentAttributeName_Result: string for attribute name of parent to put the result of the thread function in
        @param parentNextFunction: existing parent method to launch after thread completion or abortion
        """
        wx.Frame.__init__(self, parent, _id, "Searching for Orientation Matrix", size=(400, 200))

        self.threadFunctionParams = threadFunctionParams
        self.parentAttributeName_Result = parentAttributeName_Result
        self.parentNextFunction = parentNextFunction
        self.parent = parent

        # Dumb sample frame with two buttons
        self.startbtn = wx.Button(self, ID_START, "Start")
        self.stopbtn = wx.Button(self, ID_STOP, "Stop")
        self.status = wx.StaticText(self, -1, "", pos=(0, 100))

        self.startbtn.SetBackgroundColour("green")
        self.stopbtn.SetBackgroundColour("red")

        self.Bind(wx.EVT_BUTTON, self.OnStart, id=ID_START)
        self.Bind(wx.EVT_BUTTON, self.OnStop, id=ID_STOP)

        self.layout()

        # Set up event handler for any worker thread results
        EVT_RESULT(self, self.OnResult)

        # And indicate we don't have a worker thread yet
        self.worker = None

    def layout(self):
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.startbtn, 0, wx.ALL | wx.EXPAND)
        vbox.Add(self.stopbtn, 0, wx.ALL | wx.EXPAND)
        vbox.Add(self.status, 0, wx.ALL | wx.EXPAND)
        self.SetSizer(vbox)

    def OnStart(self, _):
        """Start Computation."""
        # Trigger the worker thread unless it's already busy
        if not self.worker:
            self.status.SetLabel("Computation running... Please Wait.")
            self.worker = WorkerThread(self, self.threadFunctionParams)

    def OnStop(self, _):
        """Stop Computation."""
        # Flag the worker thread to stop if running
        if self.worker:
            self.status.SetLabel("Trying to abort computation")
            self.worker.abort()

    def OnResult(self, event):
        """Show Result status."""
        if event.data is None:
            # Thread aborted (using our convention of None return)
            self.status.SetLabel("Computation aborted")
        elif event.data in (True,):
            print("finished by True event.data")

        else:
            print("finished!    triggered by event.data %s" % event.data)
            # Process results here
            self.status.SetLabel("Computation Result: %s" % event.data)

        # In either event, the worker is done
        #         print "self.parent", self.parent
        #         print "self.worker.fctOutputResults", self.worker.fctOutputResults
        setattr(self.parent, self.parentAttributeName_Result, self.worker.fctOutputResults)
        self.parentNextFunction()
        self.worker = None
        #         print "closing two seconds !"
        #         time.sleep(2)
        self.Close()


APP_SIZE_X = 300
APP_SIZE_Y = 200


class testFrame(wx.Frame):
    def __init__(self, parent, _id, title):
        wx.Frame.__init__(self, parent, _id, title, size=(APP_SIZE_X, APP_SIZE_Y))

        wx.Button(self, 1, "Close", (50, 130))
        wx.Button(self, 2, "Compute", (150, 130), (110, -1))

        self.Bind(wx.EVT_BUTTON, self.OnClose, id=1)
        self.Bind(wx.EVT_BUTTON, self.OnCompute, id=2)

        self.Centre()

    def OnClose(self, _):
        if self.TGframe is not None:
            if self.TGframe.worker is not None:
                self.TGframe.worker.abort()

        self.Close(True)

    def HandleResults(self):
        print("Results of 'OnCompute()':", self.MyResults)

    def readdata(self):
        self.data = list(range(200))

    def OnCompute(self, _):

        self.MyResults = None

        self.readdata()
        arg1 = self.data
        arg2 = self.data[5:-5]
        arg3 = 0

        fctparams = [myfunction,
            (arg1, arg2, arg3),
            {"keyarg1": "foo", "keyarg2": [1, 2, 3], "keyarg3": (0, -1, "bar")}]

        self.TGframe = ThreadHandlingFrame(self,
                                    -1,
                                    threadFunctionParams=fctparams,
                                    parentAttributeName_Result="MyResults",  # attribute where to put the output of function
                                    parentNextFunction=self.HandleResults)  # function to be called after completion of stop
        self.TGframe.OnStart(1)
        self.TGframe.startbtn.SetFocus()
        self.TGframe.Show(True)
        self.TGframe.Centre()


def myfunction(a, b, c, keyarg1=None, keyarg2=1, keyarg3=0, worker=None, otherkeyarg=True):
    """
    the function must contain the worker keyword arg and use it to communicate with client class
    Two things to do:
    - place  several 'if worker._want_abort'  conditions
    - set worker.fctOutputResults to the data you want to handle after this thread
     (parent.parentAttributeName_Result will be pointed to these data)

    """

    WORKEREXISTS = False
    if worker is not None:
        WORKEREXISTS = True

    # -------- core for computations with loop (for or while)
    res = []
    nbelem = len(b)
    print("b", b)
    NORMALRUNNING = True
    for k in range(nbelem):
        val = 2 * a[k] + b[k]
        res.append(val)
        time.sleep(0.3)
        print("val: %d  k=%d/%d" % (val, k, nbelem))

        if (
            WORKEREXISTS and worker._want_abort
        ):  # places in loops where thread can be stopped
            worker.callbackfct("ABORTED")
            NORMALRUNNING = False

            break

    if WORKEREXISTS:
        if NORMALRUNNING:
            worker.callbackfct("COMPLETED!")
        worker.fctOutputResults = res
        # worker attribute to set with results to be send to parent client class
        # (by specific attribute set by user)

    print("res", res)
    return res


if __name__ == "__main__":

    GUIApp = wx.App()
    GUIframe = testFrame(None, -1, "example GUI thread")
    GUIframe.Show()
    GUIApp.MainLoop()



r"""
Module to compute Laue Patterns from several crystals in various geometry

Main author is J. S. Micha:   micha [at] esrf [dot] fr

version July 2019
from LaueTools package for python2 hosted in

http://sourceforge.net/projects/lauetools/

or for python3 and 2 in

https://gitlab.esrf.fr/micha/lauetools
"""
import sys
import numpy as np

if sys.version_info.major == 3:
    from . import dict_LaueTools as DictLT
    from . import generaltools as GT
    from . import lauecore as LAUE
    from . import LaueGeometry as LTGeo
    from . import CrystalParameters as CP

else:
    import dict_LaueTools as DictLT
    import generaltools as GT
    import lauecore as LAUE
    import LaueGeometry as LTGeo
    import CrystalParameters as CP

try:
    WXPYTHON = True
    import wx
except ImportError:
    WXPYTHON = False

def Read_GrainListparameter(param):
    r"""
    Read dictionary of  input key parameters for simulation
    """
    # Elem, Extinc, Transf_a, Rot, Bmatrix, Transf_c, GrainName, transform = param
    key_material, Extinc, Transf_a, Rot, Bmatrix, Transf_c = param[:6]
    GrainName = str(param[6])
    # print "param in Read_GrainListparameter",param

    Extinctions = DictLT.dict_Extinc[str(Extinc)]
    Transform_labframe = DictLT.dict_Transforms[Transf_a]
    orientMatrix = DictLT.dict_Rot[Rot]
    B_matrix = DictLT.dict_Vect[Bmatrix]
    Transform_crystalframe = DictLT.dict_Transforms[Transf_c]

    return ([key_material, Extinctions, Transform_labframe, orientMatrix, B_matrix, Transform_crystalframe],
            GrainName, )


def Construct_GrainsParameters_parametric(SelectGrains_parametric):
    r"""
    return list of simulation parameters for each grain set (mother and children grains)
    """
    list_selectgrains_param = []
    # keys from dialogs were in reverse order
    # self.SelectGrains_parametric  == parametric_Grain_Dialog().SelectGrains
    for key_grain in sorted(SelectGrains_parametric.keys()):
        # print "self.SelectGrains_parametric[key_grain]",self.SelectGrains_parametric[key_grain]
        list_selectgrains_param.append(Read_GrainListparameter(SelectGrains_parametric[key_grain]))
    # print list_selectgrains_param
    return list_selectgrains_param


def dosimulation_parametric(_list_param, Transform_params=None, SelectGrains=None, emax=25.0,
                                                            emin=5.0,
                                                            detectordistance=68.7,
                                                            detectordiameter=165.0,
                                                            posCEN=(1024.0, 1024.0),
                                                            cameraAngles=(0.0, 0.0),
                                                            gauge=None,
                                                            kf_direction="Z>0",
                                                            pixelsize=165.0 / 2048,
                                                            dictmaterials=DictLT.dict_Materials):
    r"""
    Simulation of orientation or deformation gradient.
    From parent grain simulate a list of transformations (deduced by a parametric variation)

    _list_param   : list of parameters for each grain  [grain parameters, grain name]

    posCEN =(Xcen, Ycen)
    cameraAngles =(Xbet, Xgam)

    :return: (list_twicetheta,
                list_chi,
                list_energy,
                list_Miller,
                list_posX,
                list_posY,
                ParentGrainName_list,
                list_ParentGrain_transforms,
                calib,
                total_nb_grains)

    TODO:simulate for any camera position
    TODO: simulate spatial distribution of laue pattern origin
    """
    print("\n\n********* Starting dosimulation_parametric *********\n\n")

    # Number_ParentGrains parent grains
    Number_ParentGrains = len(_list_param)

    # Extracting parent grains  simluation param and name
    # creating list of simulation parameters for parent grains
    ListParam = []
    ParentGrainName_list = []
    for m in range(Number_ParentGrains):
        # print "_list_param[m]", _list_param[m]
        _paramsimul, _grainname = _list_param[m]

        elem = np.shape(np.array(_paramsimul[0]))
        # print "elem parametric",elem

        # convert EULER angles to matrix in case of  3 input elements
        if np.shape(np.array(_paramsimul[2])) == (3,):
            _paramsimul[2] = GT.fromEULERangles_toMatrix(_paramsimul[2])

        ListParam.append(_paramsimul)
        ParentGrainName_list.append(_grainname)

    #            print "ListParam in dosimulation_parametric", ListParam
    print("ParentGrainName_list", ParentGrainName_list)

    # Calculating Laue spots of each parent grain ----------------------------

    print("Doing simulation with %d parent grains" % Number_ParentGrains)
    list_twicetheta = []
    list_chi = []
    list_energy = []
    list_Miller = []
    list_posX = []
    list_posY = []

    total_nb_grains = 0

    # list of [Parent grain index,Number of corresponding transforms, transform_type]
    list_ParentGrain_transforms = []

    if gauge and WXPYTHON:
        gaugecount = 0
        gauge.SetValue(gaugecount)
        # gauge count max has been set to 1000*Number_ParentGrains

    # loop over parent grains
    for parentgrain_index in range(Number_ParentGrains):

        # read simulation parameters
        name_of_grain = ParentGrainName_list[parentgrain_index]
        print("\n\n %d, name_of_grain: %s" % (parentgrain_index, name_of_grain))

        Laue_classic_param = ListParam[parentgrain_index]

        key_material, Extinc, Ta, U, B, Tc = Laue_classic_param  # from combos

        # build GrainSimulParam
        GrainSimulParam = [0, 0, 0, 0]

        # user has entered his own B matrix
        if key_material == "inputB":
            # print "\n**************"
            # print "using Bmatrix containing lattice parameter"
            # print "****************\n"

            # take then parameters from combos
            GrainSimulParam[0] = B
            GrainSimulParam[1] = Extinc
            GrainSimulParam[2] = np.dot(np.dot(Ta, U), Tc)
            GrainSimulParam[3] = "inputB"

        # user uses a pre defined B matrix contain in material dictionnary
        # (need to read lattice parameter or element definition
        # then compute B matrix)
        elif key_material != "inputB":

            grain = CP.Prepare_Grain(key_material, np.eye(3), dictmaterials=dictmaterials)

            # print "grain in dosimulation_parametric() input Element",grain

            # B0, Extinc0, U0, key = grain  # U0 is identity
            B0, _, U0, _ = grain  # U0 is identity

            # new B matrix
            newB = np.dot(Tc, np.dot(B, B0))
            # Extinction is overwritten by value in comboExtinc
            newExtinc = Extinc
            # new U matrix
            newU = np.dot(Ta, np.dot(U, U0))

            GrainSimulParam = [newB, newExtinc, newU, key_material]

            print("Using following parameters from Material Dict.")
            print(DictLT.dict_Materials[key_material])

        # --- Simulate
        print("GrainSimulParam in dosimulation_parametric() input Element")
        print(GrainSimulParam)

        spots2pi = LAUE.getLaueSpots(DictLT.CST_ENERGYKEV / emax,
                                        DictLT.CST_ENERGYKEV / emin,
                                        [GrainSimulParam],  # bracket because of a list of one grain
                                        fastcompute=0,
                                        verbose=0,
                                        kf_direction=kf_direction,
                                        dictmaterials=dictmaterials)

        # q vectors in lauetools frame, miller indices
        # print "spots2pi",spots2pi

        # ---------  [list of 3D vectors],[list of corresponding Miller indices]

        # remove Parent Grain Laue spots too close from detector border vicinity
        Qvectors_ParentGrain, HKLs_ParentGrain = LAUE.filterQandHKLvectors(spots2pi,
                                                                    detectordistance,
                                                                    detectordiameter, kf_direction)

        # Qvectors_ParentGrain, HKLs_ParentGrain = spots2pi

        if gauge and WXPYTHON:
            gaugecount += 100
            # print "gaugecount += 100",gaugecount
            gauge.SetValue(gaugecount)
            wx.Yield()

        # --- Calculating small deviations(rotations and strain)
        # --- from parent grains according to transform

        # print " in simul Transform_params",Transform_params
        # print " in simul SelectGrains",SelectGrains

        # get transform
        if Transform_params is None:
            Transform_listparam = [""]
        elif Transform_params is not None:
            Transform_params[""] = [""]

            # print "SelectGrains[name_of_grain]",SelectGrains[name_of_grain]
            Transform_listparam = Transform_params[SelectGrains[name_of_grain][7]]
            if Transform_listparam == "":
                Transform_listparam = [""]

        # print "Transform_listparam",Transform_listparam

        nb_transforms = 1
        print("GrainSimulParam", GrainSimulParam)
        matrix_list = [np.eye(3)]

        # matrix giving a*,b*,c* in absolute x, y,z frame
        # this matrix is used for strain a*,b*,c* are NORMALIZED:
        # this is a B matrix used in  q= U B G* formalism
        # this matrix represents also an initial orientation

        # Calculates matOrient which is U*B in q = U*B*Gstar
        matOrient = np.dot(GrainSimulParam[2], GrainSimulParam[0])

        if Transform_listparam != "":
            # print "Transform_listparam[0]",Transform_listparam[0]
            if Transform_listparam[0] == "r_axis":
                axis_list = Transform_listparam[2]
                angle_list = Transform_listparam[1]
                nb_transforms = len(angle_list)

            elif Transform_listparam[0] in ("r_axis_d", "r_axis_d_slipsystem"):
                axis_list = Transform_listparam[2]
                # print "axis_list before orientation in d frame", axis_list
                angle_list = Transform_listparam[1]
                nb_transforms = len(angle_list)
                # axis coordinate change from abc frame(direct crystal) to a*b*c* frame( reciprocal crystal)
                axis_list_c = np.array([CP.fromrealframe_to_reciprocalframe(ax, GrainSimulParam[0])
                                                                    for ax in axis_list])
                #  print "axis_list_c", axis_list_c
                # axis coordinate change from a*b*c* frame(crystal) to absolute frame
                axis_list = np.dot(matOrient, axis_list_c.T).T
            #                 print "axis_list in absolute frame from d frame", axis_list

            # general transform expressed in absolute lauetools frame
            elif Transform_listparam[0] == "r_axis_c":
                axis_list = Transform_listparam[2]
                # print "axis_list before orientation in c frame",axis_list
                angle_list = Transform_listparam[1]
                nb_transforms = len(angle_list)
                # axis coordinate change from hkl frame(crystal) to absolute frame
                axis_list = np.dot(matOrient, axis_list.T).T
            #                 print "axis_list in absolute frame from c frame", axis_list

            # general transform expressed in absolute lauetools frame
            elif Transform_listparam[0] == "r_mat":
                matrix_list = Transform_listparam[1]
                nb_transforms = len(matrix_list)

                # general transform expressed in crystal frame

            elif Transform_listparam[0] == "r_mat_d":
                raise ValueError("r_mat_d matrix transform with d frame is not implemented yet")

            elif Transform_listparam[0] == "r_mat_c":
                # print "using r_mat_c"
                matrix_list = Transform_listparam[1]
                nb_transforms = len(matrix_list)
                # then convert transform in absolute lauetools frame
                for k in range(nb_transforms):
                    matrix_list[k] = np.dot(
                        matOrient, np.dot(matrix_list[k], np.linalg.inv(matOrient)))
                    # matrix_list[k] = np.dot(inv(matOrient),np.dot(matrix_list[k],matOrient))

                # transform is a list of tensile transforms
            elif isinstance(Transform_listparam[0], list):
                # is a list of 's_axis' or 's_axis_c
                liststrainframe = Transform_listparam[0]
                # list of 3 arrays, each array contains the nb_transforms axis
                # (array of three elements)
                _axis_list = Transform_listparam[2]
                # list of 3 arrays, each array contains the nb_transforms strain factor
                factor_list = Transform_listparam[1]

                # print "axis_list in c frame",_axis_list
                # print "factor_list",factor_list
                nb_transforms = len(factor_list[0])

                axis_list = [np.ones(3) for k in range(3)]
                # loop over the three proposed axial strain in simulation board
                for mm in range(3):
                    if liststrainframe[mm] == "s_axis_c":
                        axis_list[mm] = np.dot(
                            matOrient, np.transpose(_axis_list[mm])).T
                    else:
                        axis_list[mm] = _axis_list[mm]
                # print "axis_list in a frame",axis_list
                # print "Transform_listparam[2]",Transform_listparam[2]

        #         print "HKLs_ParentGrain", HKLs_ParentGrain
        #         print "nb_transforms", nb_transforms
        #         print "Transform_listparam[0]", Transform_listparam[0]

        calib = [detectordistance, posCEN[0], posCEN[1], cameraAngles[0], cameraAngles[1]]
        # -----------------------------------------------------
        # loop over child grains derived from transformation of a single parent grain
        for ChildGrain_index in range(nb_transforms):
            # Qvectors_ParentGrain is used to create Qvectors_ParentGrain for each chold grain
            # according to the transform

            # print "Qvectors_ParentGrain", Qvectors_ParentGrain

            # Geometrical transforms for each case
            # loop over reciprocal lattice vectors is done with numpy array functions

            # for rotation around axis expressed in any frame
            if Transform_listparam[0] in ("r_axis", "r_axis_c", "r_axis_d", "r_axis_d_slipsystem"):
                # print "angle, axis",angle_list[ChildGrain_index],axis_list[ChildGrain_index]
                qvectors_ChildGrain = GT.rotate_around_u(Qvectors_ParentGrain[0],
                                                            angle_list[ChildGrain_index],
                                                            u=axis_list[ChildGrain_index])
                # list of spot which are on camera(without harmonics)
                # hkl are common to all child grains
                spots2pi = [qvectors_ChildGrain], HKLs_ParentGrain

            # for general transform expressed in any frame
            elif (Transform_listparam[0] == "r_mat" or
                            Transform_listparam[0] in ("r_mat_c", "r_mat_d") or
                            Transform_listparam == "" or
                            Transform_listparam == [""]):

                # general transformation is applied to q vector
                # expressed in lauetools absolute frame
                qvectors_ChildGrain = np.dot(
                    matrix_list[ChildGrain_index], Qvectors_ParentGrain[0].T).T

                # if 0:
                #     print(" 10 first transpose(Qvectors_ParentGrain[0])",
                #         Qvectors_ParentGrain[0].T[:, :10])
                #     print("%d / %d" % (ChildGrain_index, nb_transforms))
                #     print("current matrix", matrix_list[ChildGrain_index])
                #     print(np.shape(qvectors_ChildGrain))
                #     print("GrainSimulParam", GrainSimulParam)
                #     print("qvectors_ChildGrain", qvectors_ChildGrain[:10])
                # list of spot which are on camera(without harmonics)
                spots2pi = [qvectors_ChildGrain], HKLs_ParentGrain

            # for the three consecutive axial strains
            elif isinstance(Transform_listparam[0], list):

                first_traction = GT.tensile_along_u(
                    Qvectors_ParentGrain[0],
                    factor_list[0][ChildGrain_index],
                    u=axis_list[0][ChildGrain_index])
                second_traction = GT.tensile_along_u(
                    first_traction,
                    factor_list[1][ChildGrain_index],
                    u=axis_list[1][ChildGrain_index])
                qvectors_ChildGrain = GT.tensile_along_u(
                    second_traction,
                    factor_list[2][ChildGrain_index],
                    u=axis_list[2][ChildGrain_index])
                # list of spots for a child grain (on camera + without harmonics)
                spots2pi = [qvectors_ChildGrain], HKLs_ParentGrain

            else:
                # no transformation
                pass

            # test whether there is at least one Laue spot in the camera
            for elem in spots2pi[0]:
                if len(elem) == 0:
                    print("There is at least one child grain without peaks on CCD camera for ChildGrain_index= %.3f"
                        % ChildGrain_index)
                    break

            # ---------------------------------
            # filter spots to keep those in camera, filter harmonics
            try:
                # print("kf_direction = (in dosimulationparametric)", kf_direction)
                if kf_direction == "Z>0" or isinstance(kf_direction, list):  # or isinstance(kf_direction, np.array):
                    Laue_spot_list = LAUE.filterLaueSpots(spots2pi,
                                                    fileOK=0,
                                                    fastcompute=0,
                                                    detectordistance=detectordistance,
                                                    detectordiameter=detectordiameter*1.2,  # * 1.2, # avoid losing some spots in large transformation
                                                    kf_direction=kf_direction,
                                                    HarmonicsRemoval=1,
                                                    pixelsize=pixelsize)

                    # for elem in Laue_spot_list[0][:10]:
                    # print elem

                    # print "Laue_spot_list[0][0].Twicetheta"
                    # print Laue_spot_list[0][0].Twicetheta

                    if gauge and WXPYTHON:
                        gaugecount = gaugecount + 900 / nb_transforms
                        gauge.SetValue(gaugecount)
                        wx.Yield()
                        # print "ChildGrain_index 900%nb_transforms",ChildGrain_index, gaugecount

                    Listspots = Laue_spot_list[0]
                    twicetheta = [spot.Twicetheta for spot in Listspots]
                    chi = [spot.Chi for spot in Listspots]
                    energy = [spot.EwaldRadius * DictLT.CST_ENERGYKEV for spot in Listspots]
                    Miller_ind = [list(spot.Millers) for spot in Listspots]

                    calib = [detectordistance, posCEN[0], posCEN[1], cameraAngles[0], cameraAngles[1]]

                    # print("calib parameters in dosimulation_parametric")
                    # print(calib)
                    # print("pixelsize", pixelsize)
                    # print("framedim", framedim)

                    posx, posy = LTGeo.calc_xycam_from2thetachi(twicetheta,
                                                                    chi,
                                                                    calib,
                                                                    pixelsize=pixelsize,
                                                                    kf_direction=kf_direction)[:2]
                    # vecRR = [spot.Qxyz for spot in Laue_spot_list[0]] #uf_lab in JSM LaueTools frame

                    list_twicetheta.append(twicetheta)
                    list_chi.append(chi)
                    list_energy.append(energy)
                    list_Miller.append(Miller_ind)
                    list_posX.append(posx.tolist())
                    list_posY.append(posy.tolist())

                    # success = 1

                elif kf_direction in ("Y<0", "Y>0"):
                    # TODO: patch for test: detectordistance = 126.5

                    Laue_spot_list = LAUE.filterLaueSpots(spots2pi,
                                                            fileOK=0,
                                                            fastcompute=0,
                                                            detectordistance=detectordistance,
                                                            detectordiameter=detectordiameter,  # * 1.2, # avoid losing some spots in large transformation
                                                            kf_direction=kf_direction,
                                                            HarmonicsRemoval=1,
                                                            pixelsize=pixelsize)

                    # for elem in Laue_spot_list[0][:10]:
                    # print elem

                    # print "Laue_spot_list[0][0].Twicetheta"
                    # print Laue_spot_list[0][0].Twicetheta
                    if gauge and WXPYTHON:
                        gaugecount = gaugecount + 900 / nb_transforms
                        gauge.SetValue(gaugecount)
                        wx.Yield()
                        # print "ChildGrain_index 900%nb_transforms",ChildGrain_index, gaugecount

                    twicetheta = [spot.Twicetheta for spot in Laue_spot_list[0]]
                    chi = [spot.Chi for spot in Laue_spot_list[0]]
                    energy = [spot.EwaldRadius * DictLT.CST_ENERGYKEV
                        for spot in Laue_spot_list[0]]
                    Miller_ind = [list(spot.Millers) for spot in Laue_spot_list[0]]

                    calib = [detectordistance, posCEN[0], posCEN[1], cameraAngles[0], cameraAngles[1]]

                    posx, posy = LTGeo.calc_xycam_from2thetachi(twicetheta, chi, calib,
                                        pixelsize=pixelsize,
                                        kf_direction=kf_direction)[:2]
                    # posx, posy, theta0 = LTGeo.calc_xycam_from2thetachi(twicetheta, chi, calib, pixelsize = self.pixelsize)

                    posx = [spot.Xcam for spot in Laue_spot_list[0]]
                    posy = [spot.Ycam for spot in Laue_spot_list[0]]

                    # vecRR = [
                    #     spot.Qxyz for spot in Laue_spot_list[0]
                    # ]  # uf_lab in JSM frame

                    # print "twicetheta",twicetheta
                    # print "2*th0",(2*theta0).tolist()

                    list_twicetheta.append(twicetheta)
                    list_chi.append(chi)
                    list_energy.append(energy)
                    list_Miller.append(Miller_ind)
                    list_posX.append(posx)
                    list_posY.append(posy)

                    # success = 1

                elif kf_direction in ("X>0", "X<0"):  # transmission mode or back reflection mode

                    # print("spots2pi",spots2pi)
                    Laue_spot_list = LAUE.filterLaueSpots(spots2pi,
                                                    fileOK=0,
                                                    fastcompute=0,
                                                    detectordistance=detectordistance,
                                                    detectordiameter=detectordiameter*1.2,  # * 1.2, # avoid losing some spots in large transformation
                                                    kf_direction=kf_direction,
                                                    HarmonicsRemoval=1,
                                                    pixelsize=pixelsize)

                    if Laue_spot_list is None:
                        list_twicetheta.append([])
                        list_chi.append([])
                        list_energy.append([])
                        list_Miller.append([])
                        list_posX.append([])
                        list_posY.append([])
                        continue

                    # print("Laue_spot_list",Laue_spot_list)
                    # for elem in Laue_spot_list[0]:
                    #     print('transmission spots',elem)

                    # print "Laue_spot_list[0][0].Twicetheta"
                    # print Laue_spot_list[0][0].Twicetheta
                    if gauge and WXPYTHON:
                        gaugecount = gaugecount + 900 / nb_transforms
                        gauge.SetValue(gaugecount)
                        wx.Yield()
                        # print "ChildGrain_index 900%nb_transforms",ChildGrain_index, gaugecount

                    twicetheta = [spot.Twicetheta for spot in Laue_spot_list[0]]
                    chi = [spot.Chi for spot in Laue_spot_list[0]]
                    energy = [spot.EwaldRadius * DictLT.CST_ENERGYKEV for spot in Laue_spot_list[0]]
                    Miller_ind = [list(spot.Millers) for spot in Laue_spot_list[0]]

                    posx, posy = LTGeo.calc_xycam_from2thetachi(twicetheta, chi, calib,
                                        pixelsize=pixelsize,
                                        kf_direction=kf_direction)[:2]
                    posx = posx.tolist()
                    posy = posy.tolist()

                    list_twicetheta.append(twicetheta)
                    list_chi.append(chi)
                    list_energy.append(energy)
                    list_Miller.append(Miller_ind)
                    list_posX.append(posx)
                    list_posY.append(posy)

                    # success = 1

            except UnboundLocalError:
                txt = "With theses parameters, there are no peaks in the CCD frame!!\n"
                txt += "for transform with t = %.3f\n" % ChildGrain_index
                txt += "It may seem that the transform you have designed has a too large amplitude\n"
                txt += "-Try then to reduce the variation range of t\n"
                txt += "-Or reduce ratio between extrema in input matrix transform\n\n"

                # success = 0
                break

            # end of loop over transforms(or children grains)

        transform_type = "parametric"
        print("Transform_listparam in dosimultion", Transform_listparam)

        if not isinstance(Transform_listparam[0], list):
            if Transform_listparam[0].endswith("slipsystem"):
                transform_type = "slipsystem"

        list_ParentGrain_transforms.append([parentgrain_index, nb_transforms, transform_type])
        total_nb_grains += nb_transforms

        if gauge and WXPYTHON:
            gaugecount += (parentgrain_index + 1) * 1000
            gauge.SetValue(gaugecount)
            wx.Yield()
            # print "(parentgrain_index+1)*1000",gaugecount

    # end of loop over parent grains-------------------------------

    print("total_nb_grains (cumulated from single and grains assembly):", total_nb_grains)

    # 1 grain data lists are listed i.e. use list_twicetheta[0] etc.
    # polygrain use list_twicetheta

    print("List of Grain Name", ParentGrainName_list)
    print("Number_ParentGrains of parent grains", Number_ParentGrains)
    print("Number_ParentGrains of spots in grain0", len(list_twicetheta[0]))
    print('list_ParentGrain_transforms', list_ParentGrain_transforms)

    data = (list_twicetheta,
        list_chi,
        list_energy,
        list_Miller,
        list_posX,
        list_posY,
        ParentGrainName_list,
        list_ParentGrain_transforms,
        calib,
        total_nb_grains)

    return data

if __name__ == "__main__":
    print('test of multigrainSimulator.py')



# Simple minded model objects for Laue spots


class Spot(object):
    """
    Simple minded object that represents a fitted laue spot
    """

    def __init__(self, data, listoffields):
        for val, field in zip(data, listoffields):
            self.__setattr__(field, float(val))


#            print "%s field is :" % field, self.__getattribute__(field)


#    def GetSizeInMb(self):
#        return self.sizeInBytes / (1024.0 * 1024.0)


def GetAllspots(listofdata, listoffields):
    """
    Return a list of spotscollection of tracks
    """
    #    print Spot(listofdata[0], listoffields)
    return [Spot(data, listoffields) for data in listofdata]



from setuptools import setup, find_packages

import sys
if sys.version_info.major == 3:
    import LaueTools
    python3entry_points = {
    'console_scripts': [
        'lauetools = LaueTools.LaueToolsGUI:start',
	'peaksearch = LaueTools.FileSeries.Peak_Search:start',
	'indexrefine = LaueTools.FileSeries.Index_Refine:start',
	'plotmeshGUI = LaueTools.plotmeshspecGUI:start']}
else:
  python3entry_points= {}

# this will be displayed on pypi at the front page of the project
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="LaueTools",
    version="3.0.0.56",
    packages=find_packages(),
    python_requires='>=2.6 , <3.9',

    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    install_requires=['docutils>=0.3',
                      'numpy>=1.11.3',
                      'scipy>=0.19.0',
                      'matplotlib>=2.0.0',
                      'wxpython>=3.0',
                      'networkx>=2.1'],  # add fabio ?

    include_package_data=True,


    # metadata for upload to PyPI
    author="J S Micha",
    author_email="micha@esrf.fr",
    description="Distribution of LaueTools Package from gitlab.esrf.fr repository for pip",
    long_description=long_description,
    long_description_content_type="text/markdown",
	
    license="MIT",
    keywords="Lauetools x-ray scattering data analysis GUI Laue",
    url="https://sourceforge.net/projects/lauetools/",  # project home page, if any
    classifiers=["Programming Language :: Python :: 3.8",
"Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
	"Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 2.7",
	"Programming Language :: Python :: 2.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
	entry_points = python3entry_points,
    # could also include long_description, download_url, classifiers, etc.
)




import os
import sys
import LaueTools.indexingSpotsSet as ISS

if 0:
    MainFolder = '/home/micha/LaueProjects/SiSibulle_Lukas'
    print("MainFolder", MainFolder)

    #  fields like on the GUI   FileSeries/Index_Refine.py

    filepathdat = MainFolder
    filepathcor = os.path.join(MainFolder, "corfiles")
    filepathout = os.path.join(MainFolder, "fitfiles")

    fileprefix = "LukasHR_"
    filesuffix = '.dat'
    nbdigits_filename = 4
    startindex = 0
    finalindex = 10
    stepindex = 1

    filedet = '/home/micha/LaueProjects/SiSibulle_Lukas/calibSilukas.det'

    guessedMatricesFile ='/home/micha/LaueProjects/SiSibulle_Lukas/SiSi.ubs'
    MinimumMatchingRate = 4
    fileirp = os.path.join(MainFolder, "SiSi.irp")
    spottrackingfile = '/home/micha/LaueProjects/SiSibulle_Lukas/SiLukas_acceptedspots.cor'

    nb_cpus = 3

    # boolean
    reanalyse = True
    use_previous_results = False
    updatefitfiles = False

    trackingmode= False

    verbose = 0
    
if 1:

    MainFolder = '/home/micha/LaueToolsPy3/LaueTools/Examples/CuSi'
    print("MainFolder", MainFolder)

    #  fields like on the GUI   FileSeries/Index_Refine.py

    filepathdat = os.path.join(MainFolder, "corfiles")
    filepathcor = os.path.join(MainFolder, "corfiles")
    filepathout = os.path.join(MainFolder, "fitfiles")

    fileprefix = "SiCustrain"
    filesuffix = '.cor'
    nbdigits_filename = 0
    startindex = 0
    finalindex = 5
    stepindex = 1

    filedet = None
    guessedMatricesFile = None
    MinimumMatchingRate = 4
    
    if 0:
        fileirp = os.path.join(MainFolder, "cu.irp")
        spottrackingfile = '/home/micha/LaueToolsPy3/LaueTools/Examples/CuSi/corfiles/SiCustrain5_Cu20spots_1Sipeak.fit'

    if 1:
        fileirp = os.path.join(MainFolder, "sicu.irp")
        spottrackingfile = None

    nb_cpus = 2

    # boolean
    reanalyse = True
    use_previous_results = True
    updatefitfiles = False

    trackingmode = False

    verbose = 1


multiple_results, filedict = ISS.indexFilesSeries(filepathdat, filepathcor, filepathout,
                    fileprefix, filesuffix, nbdigits_filename,
                    startindex, finalindex, stepindex,
                    filedet,
                    guessedMatricesFile, MinimumMatchingRate,
                    fileirp,
                    spottrackingfile,
                    nb_cpus,
                    reanalyse,
                    use_previous_results,
                    updatefitfiles,
                    trackingmode=trackingmode,
                    build_hdf5=True,
                    verbose=verbose)

print('nb of results', len(multiple_results))
print(multiple_results[0])



# -*- coding: utf-8 -*-
r"""
GUI class to connect spots from two sets
"""
import wx


class LinkEditor(wx.Frame):
    def __init__(self, parent, _id, title, previouslist, millerlist, intensitylist=None):
        # wx.Dialog.__init__(self, parent, id, title, size=(600,500), style=wx.DEFAULT_DIALOG_STYLE)
        #         wx.Dialog.__init__(self, parent, id, title, size=(930, 500), style=wx.OK)

        wx.Frame.__init__(self, parent, _id, title, size=(930, 500))

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        vbox1 = wx.BoxSizer(wx.VERTICAL)
        vbox2 = wx.BoxSizer(wx.VERTICAL)
        vbox3 = wx.GridSizer(2, 2, 0, 0)
        vbox4 = wx.BoxSizer(wx.VERTICAL)
        pnl1 = wx.Panel(self, -1, style=wx.SIMPLE_BORDER)
        pnl2 = wx.Panel(self, -1, style=wx.SIMPLE_BORDER)
        self.lc = wx.ListCtrl(self, -1, style=wx.LC_REPORT)
        self.lc.InsertColumn(0, "Experimental")
        self.lc.InsertColumn(1, "Simulated")
        self.lc.InsertColumn(2, "Miller")
        self.lc.InsertColumn(3, "Intensity")
        self.lc.SetColumnWidth(0, 100)
        self.lc.SetColumnWidth(1, 100)
        self.lc.SetColumnWidth(2, 150)
        self.lc.SetColumnWidth(3, 100)
        vbox1.Add(pnl1, 1, wx.EXPAND | wx.ALL, 3)
        vbox1.Add(pnl2, 1, wx.EXPAND | wx.ALL, 3)
        vbox2.Add(self.lc, 1, wx.EXPAND | wx.ALL, 3)
        self.tc1 = wx.TextCtrl(pnl1, -1)
        self.tc2 = wx.TextCtrl(pnl1, -1)
        vbox3.AddMany([(wx.StaticText(pnl1, -1, "Experimental"), 0, wx.ALIGN_CENTER),
                (self.tc1, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL),
                (wx.StaticText(pnl1, -1, "Simulated"), 0, wx.ALIGN_CENTER_HORIZONTAL),
                (self.tc2, 0)])
        pnl1.SetSizer(vbox3)
        vbox4.Add(wx.Button(pnl2, 10, "Add"), 0, wx.ALIGN_CENTER | wx.TOP, 45)
        vbox4.Add(wx.Button(pnl2, 11, "Remove"), 0, wx.ALIGN_CENTER | wx.TOP, 15)
        vbox4.Add(wx.Button(pnl2, 12, "Clear All"), 0, wx.ALIGN_CENTER | wx.TOP, 15)
        vbox4.Add(wx.Button(pnl2, wx.ID_OK, "Accept and Quit"), 0, wx.ALIGN_CENTER | wx.TOP, 15)
        pnl2.SetSizer(vbox4)
        self.Bind(wx.EVT_BUTTON, self.OnAdd, id=10)
        self.Bind(wx.EVT_BUTTON, self.OnRemove, id=11)
        self.Bind(wx.EVT_BUTTON, self.OnClear, id=12)
        self.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_OK)
        hbox.Add(vbox1, 1, wx.EXPAND)
        hbox.Add(vbox2, 1, wx.EXPAND)
        self.SetSizer(hbox)

        # must be of equal length
        self.millerlist = millerlist
        self.intensitylist = intensitylist

        # list for preparing the returned result
        self.listofpairs = []

        # pre defined links
        self.previouslist = previouslist
        if self.previouslist is not None:
            self.AddList(self.previouslist)

    #         self.ShowModal()

    def AddList(self, prelist_links):
        num_items = self.lc.GetItemCount()
        for elem in prelist_links:
            self.lc.InsertStringItem(num_items, str(int(elem[0])))
            self.lc.SetStringItem(num_items, 1, "unknown")
            # TODO: fill this field but it will change with the simulation parameter
            self.lc.SetStringItem(num_items, 2, "[%d, %d, %d]" % tuple(elem[1:4]))
            if self.intensitylist is not None:
                self.lc.SetStringItem(num_items, 3, str(self.intensitylist[num_items]))
            else:
                self.lc.SetStringItem(num_items, 3, "unknown")
            num_items += 1

    def OnAdd(self, _):
        if not self.tc1.GetValue() or not self.tc2.GetValue():
            return

        num_items = self.lc.GetItemCount()
        self.lc.InsertStringItem(num_items, self.tc1.GetValue())
        self.lc.SetStringItem(num_items, 1, self.tc2.GetValue())
        try:
            self.lc.SetStringItem(num_items, 2, str(self.ReadMiller(int(self.tc2.GetValue()))))
        except IndexError:
            wx.MessageBox("Theoretical spot of index %d does not exist ! \n Please remove it! "
                % int(self.tc2.GetValue()), "INFO")

        if self.intensitylist is not None:
            self.lc.SetStringItem(num_items, 3, str(self.intensitylist[int(self.tc1.GetValue())]))
        else:
            self.lc.SetStringItem(num_items, 3, "unknown")

        self.tc1.Clear()
        self.tc2.Clear()

    def ReadMiller(self, index):
        print("index", index)
        print("self.millerlist[index]", self.millerlist[index])
        return self.millerlist[index]
    def OnRemove(self, _):
        index = self.lc.GetFocusedItem()
        self.lc.DeleteItem(index)

    def OnClose(self, _):
        self.listofpairs = []
        self.linkExpMiller = []
        self.linkIntensity = []
        for idx in range(self.lc.GetItemCount()):

            item_exp = int(self.lc.GetItem(idx).GetText())

            sim_index = self.lc.GetItem(idx, 1).GetText()
            if sim_index != "unknown":
                item_sim = int(sim_index)
            else:
                item_sim = -1  # integer code for unknown index

            str_miller = str(self.lc.GetItem(idx, 2).GetText())
            print("str_miller", str_miller)
            if "," in str_miller:
                HKL = [float(ind) for ind in str_miller[1:-1].split(",")]
            else:
                HKL = [float(ind) for ind in str_miller[1:-1].split()]
            H, K, L = HKL

            intensity = float(self.lc.GetItem(idx, 3).GetText())

            # print [item_exp,item_sim,HKL]
            self.listofpairs.append([item_exp, item_sim])
            self.linkExpMiller.append([float(item_exp), H, K, L])
            self.linkIntensity.append(intensity)
            # print "\n\n self.listofpairs in editor class",self.listofpairs
        self.Close()
        return self.listofpairs, self.linkExpMiller, self.linkIntensity

    def OnClear(self, _):
        self.lc.DeleteAllItems()


if __name__ == "__main__":

    # test data:
    # old links between exp spots and assigned miller indices
    nb_of_exp_spots = 40
    previouslist = [[0, -1, 5, 2],
                        [1, 2, 3, 8],
                        [8, 2, 0, 2],
                        [10, -1, -1, -1],
                        [nb_of_exp_spots - 1, 0, 0, 1]]  # exp spot, h,k,l

    # some miller indices that can be picked up by theoretical spot index
    millerlist = [[-2, 0, 0],
                    [0, 2, 0],
                    [1, 1, 1],
                    [0, 0, 1],
                    [-1, 1, 1],
                    [1, 0, 0],
                    [-3, 1, 3]]

    # intensity list corresponding of exp. spots intensity
    intensitylist = [-6.2, 6.0, 10.0, -0.100, 1000.0, -56.0, 9.25, 236.0]

    class MyApp(wx.App):
        def OnInit(self):
            dia = LinkEditor(None,
                                -1,
                                "Spots Links Editor.py",
                                previouslist,
                                millerlist,
                                intensitylist=intensitylist)
            # dia.ShowModal()
            dia.Destroy()

            print("list of pairs", dia.listofpairs)
            print("list of Miller selection", dia.linkExpMiller)
            print("list of intensity selection", dia.linkIntensity)
            return True

    app = MyApp(0)
    app.MainLoop()



import wx

try:
    from SpecClient_gevent import SpecCommand
except ImportError:
    print('-- warning. spec control software and SpecClient_gevent missing ? (normal if you are not at the beamline)')

# --- ---------------  Plot limits board  parameters
class MessageCommand(wx.Dialog):
    """
    Class to command with spec
    """

    def __init__(self, parent, _id, title, sentence=None, speccommand=None, specconnection=None):
        """
        initialize board window
        """
        wx.Dialog.__init__(self, parent, _id, title, size=(400, 250))

        self.parent = parent
        #print("self.parent", self.parent)

        self.speccommand = speccommand

        txt1 = wx.StaticText(self, -1, "%s\n\n%s" % (sentence, self.speccommand))

        acceptbtn = wx.Button(self, -1, "OK")
        tospecbtn = wx.Button(self, -1, "Send to Spec")
        cancelbtn = wx.Button(self, -1, "Cancel")

        acceptbtn.Bind(wx.EVT_BUTTON, self.onAccept)
        cancelbtn.Bind(wx.EVT_BUTTON, self.onCancel)
        tospecbtn.Bind(wx.EVT_BUTTON, self.onCommandtoSpec)

        btnssizer = wx.BoxSizer(wx.HORIZONTAL)
        btnssizer.Add(acceptbtn, 0, wx.ALL)
        btnssizer.Add(cancelbtn, 0, wx.ALL)
        btnssizer.Add(tospecbtn, 0, wx.ALL)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txt1)
        vbox.Add(btnssizer)
        self.SetSizer(vbox)

    def onAccept(self, _):

        self.Close()

    def onCancel(self, _):

        # todo save old positions and make inverse mvt
        self.Close()

    def onCommandtoSpec(self, _):

        myspec = SpecCommand.SpecCommand("", "crg1:laue")

        print("Sending command : " + self.speccommand)

        myspec.executeCommand(self.speccommand)

        self.Close()